#include "string.h"
#include <vector>
#include <sstream>
#include <iostream>
#include <fstream>
#include <string>
#include <time.h>
#include "stdlib.h"
#include "math.h"


#include <sys/stat.h>//mkdir

#include "remc.h"
#include "protein.h"
#include "memory.h"
#include "error.h"
//#include "TMalign.h"
#include "protein_data.h"
//#include "k-means.h"



using namespace REFINE_NS;

//#define PI 3.14159265
//#define MAXAA      2000
//#define MAXATOM    28000
#define NNMOVE	   10    //number of move type
#define MAXLINE 2048
#define NTM 7 //number of helix

/* ---------------------------------------------------------------------- */
REMC::REMC(REFINE *refine) : Pointers(refine)
{
	xrep=NULL; // x y z coordinates
	//nrep=NULL; // number of replicas
	xtmp=NULL;  // x y z coordinates backup
	nmove=NULL; // number of each movement type
	nmove_accept=NULL; //accept number of each movements
	nmove_T_accept=NULL; // number of accept  movement of specified Temperature
	tmbrep=NULL; // tm definition of replicas
	tmerep=NULL; // tm definition of replicas
	//ntmrep=NULL;
	//energytype=NULL;//default use real energy

	seqNative=NULL;
	nresNative=NULL;
	xNative=NULL;
	yNative=NULL;
	zNative=NULL;

	  libAA3=NULL ;
	  libphi=NULL ;
	  libpsi=NULL ;
	  libloop1=NULL;
	  
	  libloop1_byloopID=NULL;
	   nLoopID=NULL;
	  DFIRELib=NULL;
}

/* ---------------------------------------------------------------------- */
REMC::~REMC()
{
	memory->sfree(nativepdbfile);
	memory->sfree(nativepdbfile_tmregion);


  memory->destroy_3d_double_array(xrep);
  if (xtmp != NULL) {
	  xtmp=NULL;
 	  memory->destroy_2d_double_array(xtmp);
  }
  memory->sfree(nmove);
  memory->sfree(nmove_accept);
  memory->sfree(nmove_T_accept);
  memory->destroy_2d_int_array(tmbrep);
  memory->destroy_2d_int_array(tmerep);
  memory->sfree(seqNative);
  memory->sfree(nresNative);
  memory->sfree(xNative);
  memory->sfree(yNative);
  memory->sfree(zNative);

  if (libAA3 != NULL) {
 	  libAA3=NULL;
 	  memory->destroy_2d_char_array(libAA3);
   }
   if (libphi != NULL) {
 	  libphi=NULL;
 	  memory->sfree(libphi);
   }
   if (libpsi != NULL) {
 	  libpsi=NULL;
 	  memory->sfree(libpsi);
   }
   if (libloop1 != NULL) {
 	  libloop1=NULL;
 	  memory->sfree(libloop1);
   }
   if (libloop1_byloopID != NULL) {
	 	  libloop1=NULL;
	 	  memory->destroy_2d_int_array(libloop1_byloopID);
   }
   	if (nLoopID != NULL) {
	   nLoopID=NULL;
 	  memory->sfree(nLoopID);
  }
   if (DFIRELib != NULL) {
	   DFIRELib=NULL;
 	  memory->sfree(DFIRELib);
   }
   

}

/* ---------------------------------------------------------------------- */
void REMC::init(int nrepli)
{

	nrep=nrepli; // number of replicas
	ntmrep=100;
	energytype=0;//default use real energy
	runmode=DEBUG;
	energymode=REALENERGY;

	weight0_loop=weight1_loop=weight2_loop=weight3_loop=weight4_loop=weight5_loop=weight6_loop=weight7_loop=b_loop=-1.0;


	xrep =  memory->create_3d_double_array(nrep,MAXATOM,3,"REMC:xrep");
	xtmp =  memory->create_2d_double_array(MAXATOM,3,"REMC:xtmp");
	nmove = (int *)memory->smalloc(NNMOVE*sizeof(int),"REMC:nmove");
	nmove_accept = (int *)memory->smalloc(NNMOVE*sizeof(int),"REMC: nmove_accept");
	tmbrep = memory->create_2d_int_array(nrep,ntmrep,"REMC:tmbrep");
	tmerep = memory->create_2d_int_array(nrep,ntmrep,"REMC:tmerep");
	nmove_T_accept=(int *)memory->smalloc(nrep*sizeof(int),"REMC: nmove_T_accept");

	nativepdbfile = (char *)memory->smalloc(MAXLINE*sizeof(char),"REMC:nativepdbfile");// native file name
	nativepdbfile_tmregion= (char *)memory->smalloc(MAXLINE*sizeof(char),"REMC:nativepdbfile_tmregion");// native file name
	seqNative=(char *)memory->smalloc(MAXAA*sizeof(char),"REMC:seqNative"); //one char amino acid  sequence of native
	nresNative = (int *)memory->smalloc(MAXAA*sizeof(int),"REMC:nresNative");
	xNative = (double *)memory->smalloc(MAXATOM*sizeof(double),"REMC:xNative");
	yNative = (double *)memory->smalloc(MAXATOM*sizeof(double),"REMC:yNative");
	zNative = (double *)memory->smalloc(MAXATOM*sizeof(double),"REMC:zNative");

	  //for loop library
	  libAA3=   memory->create_2d_char_array(MAXATOM,3,"REMC:libAA3");
	  libphi= (int *)memory->smalloc(MAXATOM*sizeof(int),"REMC:libphi");
	  libpsi= (int *)memory->smalloc(MAXATOM*sizeof(int),"REMC:libpsi");
	  libloop1= (int *)memory->smalloc(MAXAA*sizeof(int),"REMC:libloop1");
	  nLoopID= (int *)memory->smalloc((NTM+1)*sizeof(int),"REMC:nLoopID");
	  libloop1_byloopID = memory->create_2d_int_array(NTM+1,MAXAA,"REMC:libloop1_byloopID");
	  
	  
	  DFIRELib= (char *)memory->smalloc(MAXLINE*sizeof(char),"REMC:DFIRELib");// DFIRELib file name

}



/* ----------------------------------------------------------------------
     End by Hongjie Wu @2015.3.20
     transfer one loop residue
   ---------------------------------------------------------------------- */ 
