/*  protein data file */

#define PI 3.14159265
#define NAA         20  //number of aminoacids
#define NATM_MAX    14 //maximum number of atoms in one residue
#define MAXAA       800
#define MAXATOM     11200
#define bondK       1.0
#define angleK      1.0
#define ramacut    -3.0
#define Nrot        341
#define Rwater      1.40
#define NUnitSphere 960
// parameters for insertion free energy
#define InsertionC0	0.27	// parameter c0
#define InsertionC1	9.29	// parameter c1
#define InsertionC2	-0.645	// parameter c2
#define InsertionC3	0.00822	// parameter c3

/*
"G","GLY", 4,"Glycine"          N CA C O
"A","ALA", 5,"Alanine"          N CA C O CB
"C","CYS", 6,"Cysteine"         N CA C O CB SG
"S","SER", 6,"Serine"           N CA C O CB OG
"T","THR", 7,"Threonine"        N CA C O CB OG1 CG2
"P","PRO", 7,"Proline"          N CA C O CB CG  CD
"V","VAL", 7,"Valine"           N CA C O CB CG1 CG2
"L","LEU", 8,"Leucine"          N CA C O CB CG  CD1 CD2
"I","ILE", 8,"Isoleucine"       N CA C O CB CG1 CG2 CD1
"N","ASN", 8,"Asparagine"       N CA C O CB CG  OD1 ND2
"D","ASP", 8,"Aspartic acid"    N CA C O CB CG  OD1 OD2
"M","MET", 8,"Methionine",      N CA C O CB CG  SD  CE
"E","GLU", 9,"Glutamic acid"    N CA C O CB CG  CD  OE1 OE2
"Q","GLN", 9,"Glutamine"        N CA C O CB CG  CD  OE1 NE2
"K","LYS", 9,"Lysine"           N CA C O CB CG  CD  CE  NZ
"H","HIS",10,"Histidine"        N CA C O CB CG  ND1 CD2 CE1 NE2
"F","PHE",11,"Phenylalanine"    N CA C O CB CG  CD1 CD2 CE1 CE2 CZ
"R","ARG",11,"Arginine"         N CA C O CB CG  CD  NE  CZ  NH1 NH2
"Y","TYR",12,"Tyrosine"         N CA C O CB CG  CD1 CD2 CE1 CE2 CZ  OH
"W","TRP",14,"Tryptophan"       N CA C O CB CG  CD1 CD2 NE1 CE2 CE3 CZ2 CZ3 CH2
"M","MSE", 8,"SelenoMethionine",N CA C O CB CG  SE  CE
*/

static int sizeList[NAA+1]    = {   5,     6,    8,    9,   11,    4,   10,    8,     9,   8,
				    8,     8,    7,    9,   11,    6,    7,    7,    14,  12};

static char atomsList[NAA+1][NATM_MAX][4] = {
{  "N", "CA", "C", "O", "CB",   "",     "",    "",    "",    "",    "",    "",    "",    ""},//ALA
{  "N", "CA", "C", "O", "CB", "SG",     "",    "",    "",    "",    "",    "",    "",    ""},//CYS
{  "N", "CA", "C", "O", "CB", "CG",  "OD1", "OD2",    "",    "",    "",    "",    "",    ""},//ASP
{  "N", "CA", "C", "O", "CB", "CG",   "CD", "OE1", "OE2",    "",    "",    "",    "",    ""},//GLU
{  "N", "CA", "C", "O", "CB", "CG",  "CD1", "CD2", "CE1", "CE2",  "CZ",    "",    "",    ""},//PHE
{  "N", "CA", "C", "O",   "",   "",     "",    "",    "",    "",    "",    "",    "",    ""},//GLY
{  "N", "CA", "C", "O", "CB", "CG",  "ND1", "CD2", "CE1", "NE2",    "",    "",    "",    ""},//HIS
{  "N", "CA", "C", "O", "CB", "CG1", "CG2", "CD1",    "",    "",    "",    "",    "",    ""},//ILE
{  "N", "CA", "C", "O", "CB", "CG",   "CD",  "CE",  "NZ",    "",    "",    "",    "",    ""},//LYS
{  "N", "CA", "C", "O", "CB", "CG",  "CD1", "CD2",    "",    "",    "",    "",    "",    ""},//LEU
{  "N", "CA", "C", "O", "CB", "CG",   "SD",  "CE",    "",    "",    "",    "",    "",    ""},//MET
{  "N", "CA", "C", "O", "CB", "CG",  "OD1", "ND2",    "",    "",    "",    "",    "",    ""},//ASN
{  "N", "CA", "C", "O", "CB", "CG",   "CD",    "",    "",    "",    "",    "",    "",    ""},//PRO
{  "N", "CA", "C", "O", "CB", "CG",   "CD", "OE1", "NE2",    "",    "",    "",    "",    ""},//GLN
{  "N", "CA", "C", "O", "CB", "CG",   "CD",  "NE",  "CZ", "NH1", "NH2",    "",    "",    ""},//ARG
{  "N", "CA", "C", "O", "CB", "OG",     "",    "",    "",    "",    "",    "",    "",    ""},//SER
{  "N", "CA", "C", "O", "CB", "OG1", "CG2",    "",    "",    "",    "",    "",    "",    ""},//THR
{  "N", "CA", "C", "O", "CB", "CG1", "CG2",    "",    "",    "",    "",    "",    "",    ""},//VAL
{  "N", "CA", "C", "O", "CB", "CG",  "CD1", "CD2", "NE1", "CE2", "CE3", "CZ2", "CZ3", "CH2"},//TRP
{  "N", "CA", "C", "O", "CB", "CG",  "CD1", "CD2", "CE1", "CE2",  "CZ",  "OH",    "",    ""}//TYR
};

//static char sTab1[]    = {'A','C','D','E','F',
//                          'G','H','I','K','L',
//                          'M','N','P','Q','R',
//                          'S','T','V','W','Y',
//                          'E','B','Z','X'};

static char sTab3[][8] = {"ALA","CYS","ASP","GLU","PHE",
                          "GLY","HIS","ILE","LYS","LEU",
                          "MET","ASN","PRO","GLN","ARG",
                          "SER","THR","VAL","TRP","TYR",
                          "PCA","ASX","GLX","UNK"
                         };
static char sTab4[][8] = {"C","CA","CB","CD","CD1",
			  "CD2","CE","CE1","CE2","CE3",
			  "CG","CG1","CG2","CH2","CZ",
			  "CZ2","CZ3","N","ND1","ND2",
			  "NE","NE1","NE2","NH1","NH2",
			  "NZ","O","OD1","OD2","OE1",
			  "OE2","OG","OG1","OH","SG",
			  "SD"};


/** L-J potential epsilon values **/
static double sTabe[20][36]  = {{0.086000,0.109400,0.109400,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.170000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.210000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000},
{0.086000,0.109400,0.109400,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.170000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.210000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.250000,0.000000},
{0.086000,0.109400,0.109400,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.086000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.170000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.210000,0.210000,0.210000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000},
{0.086000,0.109400,0.109400,0.086000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.109400,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.170000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.210000,0.000000,0.000000,0.210000,0.210000,0.000000,0.000000,0.000000,0.000000,0.000000},
{0.086000,0.109400,0.109400,0.000000,0.086000,0.086000,0.000000,0.086000,0.086000,0.000000,0.086000,0.000000,0.000000,0.000000,0.086000,0.000000,0.000000,0.170000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.210000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000},
{0.086000,0.109400,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.170000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.210000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000},
{0.086000,0.109400,0.109400,0.000000,0.000000,0.086000,0.000000,0.086000,0.000000,0.000000,0.086000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.170000,0.170000,0.000000,0.000000,0.000000,0.170000,0.000000,0.000000,0.000000,0.210000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000},
{0.086000,0.109400,0.109400,0.000000,0.109400,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.109400,0.109400,0.000000,0.000000,0.000000,0.000000,0.170000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.210000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000},
{0.086000,0.109400,0.109400,0.109400,0.000000,0.000000,0.109400,0.000000,0.000000,0.000000,0.109400,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.170000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.170000,0.210000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000},
{0.086000,0.109400,0.109400,0.000000,0.109400,0.109400,0.000000,0.000000,0.000000,0.000000,0.109400,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.170000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.210000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000}, 
{0.086000,0.109400,0.109400,0.000000,0.000000,0.000000,0.109400,0.000000,0.000000,0.000000,0.109400,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.170000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.210000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.250000},
{0.086000,0.109400,0.109400,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.086000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.170000,0.000000,0.170000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.210000,0.210000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000},
{0.086000,0.109400,0.109400,0.109400,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.109400,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.170000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.210000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000},
{0.086000,0.109400,0.109400,0.086000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.109400,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.170000,0.000000,0.000000,0.000000,0.000000,0.170000,0.000000,0.000000,0.000000,0.210000,0.000000,0.000000,0.210000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000},
{0.086000,0.109400,0.109400,0.109400,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.109400,0.000000,0.000000,0.000000,0.086000,0.000000,0.000000,0.170000,0.000000,0.000000,0.170000,0.000000,0.000000,0.170000,0.170000,0.000000,0.210000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000},
{0.086000,0.109400,0.109400,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.170000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.210000,0.000000,0.000000,0.000000,0.000000,0.210400,0.000000,0.000000,0.000000,0.000000},
{0.086000,0.109400,0.109400,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.109400,0.000000,0.000000,0.000000,0.000000,0.170000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.210000,0.000000,0.000000,0.000000,0.000000,0.000000,0.210400,0.000000,0.000000,0.000000},
{0.086000,0.109400,0.109400,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.109400,0.109400,0.000000,0.000000,0.000000,0.000000,0.170000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.210000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000},
{0.086000,0.109400,0.109400,0.000000,0.086000,0.086000,0.000000,0.000000,0.086000,0.086000,0.086000,0.000000,0.000000,0.086000,0.000000,0.086000,0.086000,0.170000,0.000000,0.000000,0.000000,0.170000,0.000000,0.000000,0.000000,0.000000,0.210000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000},
{0.086000,0.109400,0.109400,0.000000,0.086000,0.086000,0.000000,0.086000,0.086000,0.000000,0.086000,0.000000,0.000000,0.000000,0.086000,0.000000,0.000000,0.170000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.210000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.210400,0.000000,0.000000}};

/** L-J potential sigma values **/
static double sTabs[20][36] = {{3.399670,3.399670,3.399670,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,3.250000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,2.959920,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000},
{3.399670,3.399670,3.399670,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,3.250000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,2.959920,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,3.563590,0.000000},
{3.399670,3.399670,3.399670,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,3.399670,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,3.250000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,2.959920,2.959920,2.959920,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000},
{3.399670,3.399670,3.399670,3.399670,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,3.399670,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,3.250000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,2.959920,0.000000,0.000000,2.959920,2.959920,0.000000,0.000000,0.000000,0.000000,0.000000},
{3.399670,3.399670,3.399670,0.000000,3.399670,3.399670,0.000000,3.399670,3.399670,0.000000,3.399670,0.000000,0.000000,0.000000,3.399670,0.000000,0.000000,3.250000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,2.959920,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000},
{3.399670,3.399670,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,3.250000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,2.959920,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000},
{3.399670,3.399670,3.399670,0.000000,0.000000,3.399670,0.000000,3.399670,0.000000,0.000000,3.399670,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,3.250000,3.250000,0.000000,0.000000,0.000000,3.250000,0.000000,0.000000,0.000000,2.959920,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000},
{3.399670,3.399670,3.399670,0.000000,3.399670,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,3.399670,3.399670,0.000000,0.000000,0.000000,0.000000,3.250000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,2.959920,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000},
{3.399670,3.399670,3.399670,3.399670,0.000000,0.000000,3.399670,0.000000,0.000000,0.000000,3.399670,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,3.250000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,3.250000,2.959920,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000},
{3.399670,3.399670,3.399670,0.000000,3.399670,3.399670,0.000000,0.000000,0.000000,0.000000,3.399670,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,3.250000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,2.959920,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000},
{3.399670,3.399670,3.399670,0.000000,0.000000,0.000000,3.399670,0.000000,0.000000,0.000000,3.399670,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,3.250000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,2.959920,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,3.563590},
{3.399670,3.399670,3.399670,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,3.399670,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,3.250000,0.000000,3.250000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,2.959920,2.959920,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000},
{3.399670,3.399670,3.399670,3.399670,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,3.399670,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,3.250000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,2.959920,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000},
{3.399670,3.399670,3.399670,3.399670,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,3.399670,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,3.250000,0.000000,0.000000,0.000000,0.000000,3.250000,0.000000,0.000000,0.000000,2.959920,0.000000,0.000000,2.959920,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000},
{3.399670,3.399670,3.399670,3.399670,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,3.399670,0.000000,0.000000,0.000000,3.399670,0.000000,0.000000,3.250000,0.000000,0.000000,3.250000,0.000000,0.000000,3.250000,3.250000,0.000000,2.959920,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000},
{3.399670,3.399670,3.399670,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,3.250000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,2.959920,0.000000,0.000000,0.000000,0.000000,3.066470,0.000000,0.000000,0.000000,0.000000},
{3.399670,3.399670,3.399670,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,3.399670,0.000000,0.000000,0.000000,0.000000,3.250000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,2.959920,0.000000,0.000000,0.000000,0.000000,0.000000,3.066470,0.000000,0.000000,0.000000},
{3.399670,3.399670,3.399670,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,3.399670,3.399670,0.000000,0.000000,0.000000,0.000000,3.250000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,2.959920,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000},
{3.399670,3.399670,3.399670,0.000000,3.399670,3.399670,0.000000,0.000000,3.399670,3.399670,3.399670,0.000000,0.000000,3.399670,0.000000,3.399670,3.399670,3.250000,0.000000,0.000000,0.000000,3.250000,0.000000,0.000000,0.000000,0.000000,2.959920,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000},
{3.399670,3.399670,3.399670,0.000000,3.399670,3.399670,0.000000,3.399670,3.399670,0.000000,3.399670,0.000000,0.000000,0.000000,3.399670,0.000000,0.000000,3.250000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,2.959920,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,3.066470,0.000000,0.000000}};
 
/************************************************************************/
/*    atom type table      */
//static int Tab6[2][2] ={{ 2,23},{ 4,15}};

static int Tab5[20][36] ={{  3,  2,  5,999,999,999,999,999,999,999,999,999,999,999,999,999,999,  1,999,999,999,999,999,999,999,999,  4,999,999,999,999,999,999,999,999,999},
			  {  8,  7, 10,999,999,999,999,999,999,999,999,999,999,999,999,999,999,  6,999,999,999,999,999,999,999,999,  9,999,999,999,999,999,999,999, 11,999},
			  { 14, 13, 16,999,999,999,999,999,999,999, 17,999,999,999,999,999,999, 12,999,999,999,999,999,999,999,999, 15, 18, 18,999,999,999,999,999,999,999},
			  { 21, 20, 23, 25,999,999,999,999,999,999, 24,999,999,999,999,999,999, 19,999,999,999,999,999,999,999,999, 22,999,999, 26, 26,999,999,999,999,999},
			  { 29, 28, 31,999, 33, 33,999, 34, 34,999, 32,999,999,999, 35,999,999, 27,999,999,999,999,999,999,999,999, 30,999,999,999,999,999,999,999,999,999},
			  { 38, 37,999,999,999,999,999,999,999,999,999,999,999,999,999,999,999, 36,999,999,999,999,999,999,999,999, 39,999,999,999,999,999,999,999,999,999},
			  { 42, 41, 44,999,999, 47,999, 48,999,999, 45,999,999,999,999,999,999, 40, 46,999,999,999, 49,999,999,999, 43,999,999,999,999,999,999,999,999,999},
			  { 52, 51, 54,999, 57,999,999,999,999,999,999, 55, 56,999,999,999,999, 50,999,999,999,999,999,999,999,999, 53,999,999,999,999,999,999,999,999,999},
			  { 60, 59, 62, 64,999,999, 65,999,999,999, 63,999,999,999,999,999,999, 58,999,999,999,999,999,999,999, 66, 61,999,999,999,999,999,999,999,999,999},
			  { 69, 68, 71,999, 73, 73,999,999,999,999, 72,999,999,999,999,999,999, 67,999,999,999,999,999,999,999,999, 70,999,999,999,999,999,999,999,999,999},
			  { 76, 75, 78,999,999,999, 81,999,999,999, 79,999,999,999,999,999,999, 74,999,999,999,999,999,999,999,999, 77,999,999,999,999,999,999,999,999, 80},
			  { 84, 83, 86,999,999,999,999,999,999,999, 87,999,999,999,999,999,999, 82,999, 89,999,999,999,999,999,999, 85, 88,999,999,999,999,999,999,999,999},
			  { 92, 91, 94, 96,999,999,999,999,999,999, 95,999,999,999,999,999,999, 90,999,999,999,999,999,999,999,999, 93,999,999,999,999,999,999,999,999,999},
			  { 99, 98,101,103,999,999,999,999,999,999,102,999,999,999,999,999,999, 97,999,999,999,999,105,999,999,999,100,999,999,104,999,999,999,999,999,999},
			  {108,107,110,112,999,999,999,999,999,999,111,999,999,999,114,999,999,106,999,999,113,999,999,115,115,999,109,999,999,999,999,999,999,999,999,999},
			  {118,117,120,999,999,999,999,999,999,999,999,999,999,999,999,999,999,116,999,999,999,999,999,999,999,999,119,999,999,999,999,121,999,999,999,999},
			  {124,123,126,999,999,999,999,999,999,999,999,999,128,999,999,999,999,122,999,999,999,999,999,999,999,999,125,999,999,999,999,999,127,999,999,999},
			  {131,130,133,999,999,999,999,999,999,999,999,134,134,999,999,999,999,129,999,999,999,999,999,999,999,999,132,999,999,999,999,999,999,999,999,999},
			  {137,136,139,999,141,142,999,999,144,145,140,999,999,148,999,146,147,135,999,999,999,143,999,999,999,999,138,999,999,999,999,999,999,999,999,999},
			  {151,150,153,999,155,155,999,156,156,999,154,999,999,999,157,999,999,149,999,999,999,999,999,999,999,999,152,999,999,999,999,999,999,158,999,999}};

/*    rotamer list      */
static int rotamerList[NAA+1][2]    = {
{  0,0},     //ALA
{  108,3},   //CYS
{  99,9},    //ASP
{  147,27},  //GLU
{  309,6},   //PHE
{  0,0},     //GLY
{  174,9},   //HIS
{  183,9},   //ILE
{  201,81},  //LYS
{  192,9},   //LEU
{  282,27},  //MET
{  81,18},   //ASN
{  315,2},   //PRO
{  111,36},  //GLN
{  0,81},    //ARG
{  317,3},   //SER
{  320,3},   //THR
{  338,3},   //VAL
{  323,9},   //TRP
{  332,6}   //TYR
};
/*    backbone independent rotamer library      */
static double rotamer[Nrot][8] = {
{    55.4,    19.8,    79.7,    16.1,    62.4,    15.0,    82.3,      11.9},
{    59.2,    23.2,    85.4,    16.6,    68.2,    15.4,  -166.2,      25.4},
{    54.5,    21.5,    79.4,    17.2,    64.6,    16.6,  -103.2,      12.5},
{    54.5,    19.0,    86.6,    13.0,   178.2,    13.1,    85.5,      12.6},
{    53.9,    18.9,    87.8,    13.9,  -178.5,    18.0,   178.1,      22.0},
{    53.5,    20.2,    87.8,    11.8,  -176.0,    17.5,   -81.1,      12.2},
{    54.5,    21.5,    83.5,    14.5,   -78.4,    20.8,   102.9,      14.2},
{    50.0,    23.4,    82.4,    13.6,   -92.5,    15.5,   169.0,      21.6},
{    54.5,    21.5,    83.5,    14.5,   -77.1,    19.2,   -84.1,      10.7},
{    62.0,    14.2,   177.8,    15.7,    64.9,    12.0,    84.4,      12.2},
{    64.1,    10.6,  -175.8,    15.2,    68.0,    15.3,  -172.6,      22.0},
{    65.1,     9.4,  -178.6,    11.3,    61.1,    13.6,  -104.5,      10.5},
{    65.3,     9.0,   180.0,    11.2,   178.0,    13.7,    85.7,      11.8},
{    66.5,    11.2,  -177.6,    13.9,  -178.9,    13.0,  -177.2,      21.2},
{    66.4,     9.1,  -175.2,    12.6,  -177.5,    11.9,   -82.5,      15.2},
{    63.8,    13.0,  -171.9,    17.8,   -64.5,    16.4,   102.9,      11.6},
{    63.4,    11.4,  -176.5,    14.2,   -66.7,    13.7,   173.2,      22.5},
{    62.9,    10.6,   179.9,    15.6,   -69.2,    13.0,   -82.5,      12.8},
{    65.2,    23.2,   -85.1,    20.9,    84.2,    18.8,    84.5,      13.8},
{    63.2,    22.7,   -85.5,    21.4,    80.7,    19.3,  -172.0,      22.0},
{    63.6,    23.8,   -82.1,    19.5,    71.6,    19.4,  -103.2,      12.5},
{    63.6,    23.8,   -78.2,    18.9,  -179.8,    16.6,    85.7,      14.3},
{    57.6,    25.1,   -83.1,    19.9,  -176.1,    13.8,  -178.4,      21.2},
{    68.7,    21.8,   -75.7,    16.5,  -177.8,     8.5,   -85.7,      13.1},
{    63.6,    23.8,   -77.9,    19.7,   -77.1,    21.6,   102.9,      14.2},
{    63.7,    23.4,   -78.1,    19.1,   -64.2,    11.7,   169.9,      22.2},
{    63.8,    22.0,   -78.2,    18.5,   -59.5,     9.1,   -83.3,      10.3},
{  -176.5,    13.7,    65.8,    12.2,    60.2,    12.7,    80.8,      15.8},
{   179.4,    11.3,    65.8,    11.5,    65.3,    11.5,  -170.9,      18.6},
{  -173.6,    13.0,    68.1,    11.0,    67.3,    18.7,  -103.0,      11.8},
{   179.3,     9.2,    66.2,    11.3,  -178.6,     8.4,    85.1,      12.0},
{  -177.0,    11.6,    67.4,    14.0,   177.0,    16.0,   171.0,      20.7},
{   178.1,    10.9,    66.9,    10.9,  -178.7,    10.8,   -85.3,      12.4},
{  -176.1,    14.0,    81.2,    15.7,   -80.9,    21.4,   103.3,      13.6},
{  -175.6,    15.8,    79.6,    16.3,   -90.4,    17.2,   172.4,      22.2},
{  -178.7,     9.9,    78.6,    13.2,   -78.3,    15.9,   -84.6,      11.1},
{  -177.3,     9.5,   178.6,    14.7,    64.9,    13.3,    80.8,      12.5},
{  -174.1,    13.6,   176.8,    16.7,    67.8,    13.7,  -167.7,      22.5},
{  -176.7,    13.1,   178.8,    15.2,    63.4,    16.4,  -103.7,      10.3},
{  -176.8,    10.2,   177.6,    11.8,   178.8,    15.4,    85.2,      13.7},
{  -176.9,     9.6,   176.3,    11.3,   178.8,    14.1,  -179.4,      21.4},
{  -177.5,     9.6,   176.5,    10.8,  -177.8,    11.7,   -85.2,      11.6},
{  -174.4,    14.8,   178.1,    12.8,   -63.4,    19.3,   105.1,      13.3},
{  -175.4,    14.5,   179.7,    17.5,   -64.8,    16.7,   165.2,      23.7},
{  -175.6,    11.4,   179.6,    11.9,   -65.0,    12.1,   -82.6,      10.8},
{  -169.7,    19.6,   -83.6,    16.7,    80.9,    16.4,    82.5,      12.8},
{  -166.0,    22.1,   -82.2,    16.5,    78.0,    18.7,  -172.4,      22.4},
{  -169.7,    19.6,   -83.6,    16.7,    71.6,    19.4,  -103.2,      12.5},
{  -171.4,    17.6,   -85.4,    10.5,  -174.0,    19.3,    84.2,      13.5},
{  -171.3,    19.4,   -87.2,    12.2,  -179.0,    15.0,  -176.1,      16.1},
{  -171.0,    15.3,   -84.3,    11.6,  -176.8,     9.2,   -81.1,      11.9},
{  -170.0,    17.5,   -92.9,    16.3,   -72.5,    21.3,   104.0,      12.6},
{  -170.2,    21.5,   -91.1,    15.0,   -67.4,    13.7,   170.0,      22.7},
{  -165.9,    19.0,   -84.4,    17.4,   -59.4,    10.4,   -82.3,      10.5},
{   -79.0,    15.9,    87.7,    20.1,    66.3,    21.7,    84.2,      12.8},
{   -83.1,    21.5,    81.9,    22.3,    69.8,    15.1,  -178.0,      22.8},
{   -80.8,    17.2,    86.2,    21.5,    67.4,    18.2,  -101.7,      12.9},
{   -78.6,    19.7,    73.8,    15.9,  -179.4,    13.6,    85.1,      16.8},
{   -85.8,    12.6,    70.4,    15.8,   174.6,    13.0,   179.0,      23.2},
{   -80.8,    15.4,    79.3,    17.9,   177.3,    13.4,   -81.8,      13.9},
{   -82.6,    17.0,    77.9,    17.0,   -78.4,    20.8,   102.9,      14.2},
{   -85.1,    16.5,    77.1,    15.6,   -91.5,    15.3,   168.7,      22.1},
{   -81.6,    16.5,    78.0,    16.2,   -79.6,    20.0,   -84.1,      10.7},
{   -66.2,    10.5,   179.5,    12.2,    66.1,    11.4,    84.3,      12.1},
{   -66.7,    12.0,   178.5,    16.5,    65.6,    12.8,  -170.1,      21.3},
{   -67.2,     9.7,  -178.5,    12.1,    63.1,    19.5,  -101.8,      15.4},
{   -66.9,     9.3,   179.7,    13.6,   178.7,    14.6,    86.7,      15.6},
{   -67.7,     9.7,   179.1,    13.3,  -179.0,    14.6,   176.3,      21.0},
{   -68.1,     9.6,  -179.7,    12.7,  -176.3,    13.9,   -82.7,      11.9},
{   -68.9,    12.3,  -175.6,     9.4,   -68.5,    15.4,   102.2,      10.1},
{   -67.0,     9.7,   179.5,    14.4,   -67.9,    13.7,   173.2,      22.6},
{   -68.4,    10.6,  -170.6,    11.1,   -64.5,    13.0,   -85.4,      10.5},
{   -63.9,    11.1,   -76.1,    14.3,    80.0,    15.3,    79.6,      11.9},
{   -64.3,    11.4,   -80.3,    17.9,    78.7,    19.3,  -165.9,      21.7},
{   -62.5,     9.3,   -80.5,    15.3,    71.6,    17.1,  -105.6,      11.9},
{   -62.6,     9.4,   -68.9,    12.4,   179.4,    15.8,    83.8,      14.3},
{   -62.7,     8.8,   -72.4,    15.0,  -177.9,    15.0,  -177.8,      20.4},
{   -63.2,     8.0,   -69.8,    10.8,  -175.2,     8.8,   -87.0,      13.2},
{   -64.9,    15.0,   -74.5,    15.1,   -79.7,    20.6,    95.0,      24.6},
{   -63.6,    12.9,   -67.9,    15.2,   -65.0,    12.3,   169.4,      22.2},
{   -62.8,     9.6,   -68.8,    12.2,   -60.8,     9.5,   -83.5,      10.0},
{    50.3,    14.4,  -115.0,    20.4,     0.0,   999.9,     0.0,     999.9},
{    64.5,    13.3,   -53.9,    15.2,     0.0,   999.9,     0.0,     999.9},
{    64.6,     7.7,     1.5,    16.3,     0.0,   999.9,     0.0,     999.9},
{    60.0,     8.9,    57.0,    16.6,     0.0,   999.9,     0.0,     999.9},
{    68.9,    13.0,   101.9,    12.0,     0.0,   999.9,     0.0,     999.9},
{    69.3,     5.0,   175.1,    16.5,     0.0,   999.9,     0.0,     999.9},
{  -176.0,    14.2,  -114.6,    19.9,     0.0,   999.9,     0.0,     999.9},
{  -173.5,    11.6,   -66.3,    13.6,     0.0,   999.9,     0.0,     999.9},
{  -170.8,    10.4,   -20.2,    12.7,     0.0,   999.9,     0.0,     999.9},
{  -169.3,    11.6,    24.3,    12.3,     0.0,   999.9,     0.0,     999.9},
{  -171.9,    11.8,    60.6,    10.8,     0.0,   999.9,     0.0,     999.9},
{  -160.3,    15.0,   121.5,    28.1,     0.0,   999.9,     0.0,     999.9},
{   -77.3,    12.3,  -137.6,    24.1,     0.0,   999.9,     0.0,     999.9},
{   -65.7,    11.1,   -63.9,    13.8,     0.0,   999.9,     0.0,     999.9},
{   -70.4,     8.7,   -23.2,    13.7,     0.0,   999.9,     0.0,     999.9},
{   -79.5,    14.5,    34.3,    13.9,     0.0,   999.9,     0.0,     999.9},
{   -65.2,    13.9,    96.5,    17.4,     0.0,   999.9,     0.0,     999.9},
{   -64.7,     9.5,   139.8,    15.9,     0.0,   999.9,     0.0,     999.9},
{    57.1,    11.5,    54.3,    18.3,     0.0,   999.9,     0.0,     999.9},
{    62.5,     7.6,     1.3,    14.7,     0.0,   999.9,     0.0,     999.9},
{    62.4,    10.6,   -55.8,    16.0,     0.0,   999.9,     0.0,     999.9},
{  -173.6,    12.3,    58.5,    16.3,     0.0,   999.9,     0.0,     999.9},
{  -172.3,    11.0,    -0.8,    16.1,     0.0,   999.9,     0.0,     999.9},
{  -167.9,    12.8,   -55.7,    18.3,     0.0,   999.9,     0.0,     999.9},
{   -69.1,    19.5,    65.0,    17.4,     0.0,   999.9,     0.0,     999.9},
{   -72.1,     8.2,   -12.7,    11.2,     0.0,   999.9,     0.0,     999.9},
{   -65.4,    10.1,   -49.8,    14.6,     0.0,   999.9,     0.0,     999.9},
{    63.7,    11.5,     0.0,   999.9,     0.0,   999.9,     0.0,     999.9},
{  -177.3,    11.5,     0.0,   999.9,     0.0,   999.9,     0.0,     999.9},
{   -64.3,    10.6,     0.0,   999.9,     0.0,   999.9,     0.0,     999.9},
{    64.1,    13.0,    74.9,    16.9,  -124.0,    19.6,     0.0,     999.9},
{    64.9,    13.6,    86.9,    14.9,   -33.0,    25.6,     0.0,     999.9},
{    62.5,    11.4,    84.6,     8.0,    38.5,    20.0,     0.0,     999.9},
{    67.7,    14.6,    84.8,    16.8,   117.2,    26.4,     0.0,     999.9},
{    61.5,    13.9,  -174.9,    17.4,  -179.8,    26.7,     0.0,     999.9},
{    64.0,     9.8,  -177.6,    12.1,   -79.1,    23.4,     0.0,     999.9},
{    65.1,    12.2,  -177.5,    14.4,    -4.1,    27.0,     0.0,     999.9},
{    65.4,    10.8,  -177.1,    13.2,    79.5,    22.4,     0.0,     999.9},
{    64.1,    13.3,   -82.4,    14.6,  -111.9,    29.4,     0.0,     999.9},
{    66.3,    13.5,   -76.0,    12.9,   -34.4,    21.6,     0.0,     999.9},
{    64.8,     9.5,   -89.6,     5.9,    38.2,    17.2,     0.0,     999.9},
{    57.6,    19.5,   -79.7,    16.2,   123.9,    19.5,     0.0,     999.9},
{  -174.0,    11.3,    60.6,    10.5,  -124.1,    20.4,     0.0,     999.9},
{  -175.4,    15.2,    74.1,    12.7,   -37.5,    28.2,     0.0,     999.9},
{  -176.1,    10.7,    65.7,     9.7,    44.6,    19.5,     0.0,     999.9},
{  -178.1,    13.6,    70.4,    12.0,   109.0,    21.7,     0.0,     999.9},
{  -175.5,    10.3,   178.6,    13.1,  -179.1,    23.0,     0.0,     999.9},
{  -178.0,    10.8,   176.1,    12.9,   -77.6,    25.4,     0.0,     999.9},
{  -177.0,     9.6,   177.3,    12.2,     1.0,    25.4,     0.0,     999.9},
{  -176.6,     9.8,  -179.4,    15.0,    74.8,    23.1,     0.0,     999.9},
{  -165.2,    15.1,   -77.7,    19.8,  -112.0,    27.8,     0.0,     999.9},
{  -169.4,    14.9,   -81.4,    11.1,   -34.9,    17.3,     0.0,     999.9},
{  -165.7,    14.5,   -83.7,    19.7,    46.1,    24.3,     0.0,     999.9},
{  -170.6,    16.7,   -75.8,     9.9,   125.3,    17.6,     0.0,     999.9},
{   -70.8,    11.8,    73.4,    15.6,  -123.8,    16.4,     0.0,     999.9},
{   -64.6,    11.7,    86.7,    19.0,   -35.4,    22.0,     0.0,     999.9},
{   -68.4,    12.1,    74.8,    12.2,    37.4,    18.5,     0.0,     999.9},
{   -72.0,    17.6,    83.7,    17.3,   110.1,    18.7,     0.0,     999.9},
{   -66.9,     9.8,   179.0,    12.7,  -177.9,    26.3,     0.0,     999.9},
{   -66.8,    10.5,  -179.7,    13.8,   -76.8,    24.3,     0.0,     999.9},
{   -67.5,     9.6,   177.6,    12.7,    -1.1,    25.8,     0.0,     999.9},
{   -65.8,     9.2,  -178.9,    14.0,    75.3,    22.6,     0.0,     999.9},
{   -66.1,    15.0,   -82.1,    10.6,  -111.8,    29.1,     0.0,     999.9},
{   -63.4,     9.4,   -63.2,    10.6,   -46.3,    19.1,     0.0,     999.9},
{   -67.0,    13.4,   -76.5,    13.1,    44.4,    26.9,     0.0,     999.9},
{   -68.5,    12.8,   -61.2,    15.2,   119.9,    17.7,     0.0,     999.9},
{    54.3,    17.7,    83.1,    11.1,    49.0,    15.9,     0.0,     999.9},
{    54.5,    20.3,    88.3,    11.9,     6.4,    14.6,     0.0,     999.9},
{    50.8,    21.1,    78.6,    18.2,   -61.4,    18.2,     0.0,     999.9},
{    64.2,    13.0,  -178.4,    16.9,    58.7,    16.6,     0.0,     999.9},
{    64.6,    13.5,  -177.4,    12.4,     1.7,    15.4,     0.0,     999.9},
{    65.3,    13.4,  -176.6,    14.6,   -58.5,    17.2,     0.0,     999.9},
{    58.6,    11.1,   -93.4,     8.7,    51.0,    16.7,     0.0,     999.9},
{    68.3,    12.8,   -81.5,     9.1,     2.6,    16.4,     0.0,     999.9},
{    65.1,    19.8,   -74.4,    13.7,   -51.1,    17.4,     0.0,     999.9},
{  -172.8,    14.1,    62.1,    13.8,    52.1,    15.4,     0.0,     999.9},
{  -178.4,    10.6,    67.8,     9.7,     9.7,    13.7,     0.0,     999.9},
{  -174.2,    17.1,    72.2,    19.5,   -65.1,    16.3,     0.0,     999.9},
{  -177.6,    12.1,   175.8,    14.3,    57.3,    17.4,     0.0,     999.9},
{  -177.3,    10.6,   177.9,    12.0,    -0.2,    15.9,     0.0,     999.9},
{  -175.2,    13.4,   179.2,    14.2,   -57.4,    17.0,     0.0,     999.9},
{  -155.8,    19.5,   -76.6,    27.9,    63.4,    18.4,     0.0,     999.9},
{  -169.0,    14.3,   -86.3,    12.5,   -14.5,    13.8,     0.0,     999.9},
{  -166.0,    19.3,   -80.1,    15.4,   -49.5,    14.5,     0.0,     999.9},
{   -66.3,    18.6,    71.1,    17.2,    52.3,    15.4,     0.0,     999.9},
{   -66.2,     9.8,    82.2,     8.0,     1.9,    15.7,     0.0,     999.9},
{   -66.8,    18.3,    91.0,    17.5,   -54.1,    17.6,     0.0,     999.9},
{   -68.3,    12.6,   179.3,    16.1,    56.0,    16.9,     0.0,     999.9},
{   -67.7,     9.1,   177.8,    12.0,    -2.2,    15.2,     0.0,     999.9},
{   -65.3,    11.5,  -178.2,    14.0,   -54.7,    16.7,     0.0,     999.9},
{   -64.1,    13.7,   -79.0,    16.9,    64.0,    18.1,     0.0,     999.9},
{   -64.2,    10.9,   -71.2,    11.2,    -9.8,    14.3,     0.0,     999.9},
{   -67.7,    11.1,   -60.7,    11.6,   -52.4,    13.9,     0.0,     999.9},
{    63.0,    10.9,    85.6,    15.5,     0.0,   999.9,     0.0,     999.9},
{    64.8,     8.7,   166.2,    32.8,     0.0,   999.9,     0.0,     999.9},
{    65.4,    11.1,   -79.5,    15.5,     0.0,   999.9,     0.0,     999.9},
{  -178.4,    11.3,    71.5,    16.7,     0.0,   999.9,     0.0,     999.9},
{  -172.8,    11.4,  -170.3,    30.6,     0.0,   999.9,     0.0,     999.9},
{  -172.6,    11.6,   -83.8,    19.8,     0.0,   999.9,     0.0,     999.9},
{   -66.0,    10.9,    85.6,    18.9,     0.0,   999.9,     0.0,     999.9},
{   -68.1,     8.6,   168.7,    27.7,     0.0,   999.9,     0.0,     999.9},
{   -63.7,    10.9,   -72.9,    19.4,     0.0,   999.9,     0.0,     999.9},
{    60.6,    17.7,    85.4,    16.1,     0.0,   999.9,     0.0,     999.9},
{    61.1,     6.6,   171.0,     8.1,     0.0,   999.9,     0.0,     999.9},
{    61.7,     8.8,   -80.0,     9.2,     0.0,   999.9,     0.0,     999.9},
{  -167.9,    11.5,    66.8,    12.1,     0.0,   999.9,     0.0,     999.9},
{  -171.4,     9.7,   167.4,     7.8,     0.0,   999.9,     0.0,     999.9},
{  -165.5,    25.8,   -78.0,    16.1,     0.0,   999.9,     0.0,     999.9},
{   -67.1,    15.1,    83.0,    19.5,     0.0,   999.9,     0.0,     999.9},
{   -63.8,     6.6,   170.1,     8.4,     0.0,   999.9,     0.0,     999.9},
{   -59.2,     7.5,   -61.0,     9.2,     0.0,   999.9,     0.0,     999.9},
{    58.7,    14.5,    80.7,    17.6,     0.0,   999.9,     0.0,     999.9},
{    71.8,    13.9,   164.6,    14.8,     0.0,   999.9,     0.0,     999.9},
{    58.2,    13.7,   -73.6,    12.4,     0.0,   999.9,     0.0,     999.9},
{  -179.2,     9.9,    63.0,     8.1,     0.0,   999.9,     0.0,     999.9},
{  -164.9,    17.4,   170.7,    27.0,     0.0,   999.9,     0.0,     999.9},
{  -169.8,    14.2,   -73.1,    18.1,     0.0,   999.9,     0.0,     999.9},
{   -86.1,    14.9,    54.6,    24.5,     0.0,   999.9,     0.0,     999.9},
{   -64.7,     9.1,   174.5,     8.6,     0.0,   999.9,     0.0,     999.9},
{   -81.5,    12.8,   -55.0,    19.6,     0.0,   999.9,     0.0,     999.9},
{    66.8,    27.0,    76.3,    19.5,    66.4,    13.3,    71.1,      19.1},
{    69.9,    26.0,    76.7,    19.3,    66.8,    19.9,   175.9,      16.7},
{    57.5,    32.2,    74.0,    18.9,    85.8,    21.5,   -83.1,      19.7},
{    65.7,    24.6,    78.2,    16.6,   174.6,    18.0,    62.6,      17.7},
{    68.9,    25.1,    76.7,    16.9,   177.2,    15.8,   175.3,      17.5},
{    68.9,    23.4,    77.8,    16.7,  -177.5,    18.4,   -68.4,      20.5},
{    66.8,    27.0,    82.3,    21.6,   -86.0,    21.8,    74.7,      24.0},
{    69.5,    27.1,    85.2,    22.6,   -93.8,    17.1,  -177.2,      17.1},
{    66.8,    27.0,    82.3,    21.6,   -83.6,    20.3,   -67.0,      17.5},
{    65.8,    12.5,   175.6,    11.0,    68.3,    21.8,    68.9,      15.9},
{    64.7,    13.2,   175.2,    12.3,    70.9,    16.9,   174.9,      15.8},
{    68.4,    18.2,  -179.3,    13.0,    87.4,    16.5,   -84.8,      17.8},
{    65.9,    11.4,  -178.1,    14.9,   173.2,    16.2,    66.9,      20.0},
{    66.0,    10.7,  -179.4,    11.3,  -179.5,    13.4,   179.9,      13.3},
{    65.6,     8.0,  -178.1,    13.2,  -175.9,    16.9,   -68.7,      17.4},
{    67.2,    13.4,  -175.2,    16.8,   -71.5,    19.7,    74.3,      21.7},
{    64.3,    16.1,  -172.4,    17.0,   -68.2,    18.2,  -176.8,      18.3},
{    67.2,    14.2,  -173.7,    14.9,   -70.3,    15.1,   -68.0,      15.2},
{    67.2,    20.0,   -83.2,    22.0,    80.8,    20.9,    69.9,      18.5},
{    64.1,    21.8,   -83.7,    22.4,    88.2,    21.4,   177.6,      16.4},
{    66.9,    21.0,   -81.3,    22.2,    88.3,    19.8,   -80.2,      19.2},
{    71.8,    22.2,   -79.7,    20.6,  -178.6,    18.0,    65.4,      16.6},
{    69.4,    21.0,   -80.1,    22.4,  -178.3,    16.7,   177.0,      18.3},
{    64.2,    18.4,   -81.9,    20.6,   174.4,    23.5,   -65.5,      20.6},
{    66.9,    21.0,   -76.7,    22.5,   -78.3,    24.7,    74.7,      24.0},
{    63.7,    20.2,   -77.8,    23.3,   -67.7,    14.3,   179.7,      16.3},
{    67.5,    20.1,   -77.5,    21.6,   -75.5,    16.9,   -67.1,      16.7},
{  -178.7,    20.1,    68.1,    16.9,    65.3,    12.9,    66.2,      18.5},
{  -172.9,    16.8,    65.9,    16.8,    71.0,    16.4,   179.7,      17.8},
{  -179.1,    14.5,    68.7,    19.9,    81.0,    23.8,   -81.3,      19.5},
{  -178.4,    12.7,    67.8,    14.2,   170.7,    17.5,    67.0,      19.4},
{  -177.7,    12.7,    70.3,    13.8,   175.8,    13.2,   179.3,      15.8},
{  -174.2,    16.5,    66.7,    16.1,  -178.3,    19.6,   -65.6,      18.6},
{  -170.3,    18.9,    87.2,    18.9,   -90.8,    20.6,    76.8,      22.6},
{  -172.6,    13.2,    88.2,    18.9,   -91.5,    18.2,  -177.5,      24.9},
{  -173.8,    16.1,    86.5,    15.9,   -86.4,    18.9,   -69.3,      18.5},
{  -174.3,    16.1,   174.4,    17.9,    72.8,    21.4,    70.5,      20.6},
{  -177.5,    12.2,   173.7,    13.6,    71.5,    18.7,   177.8,      17.0},
{  -172.2,    14.5,   168.9,    18.9,    87.1,    21.4,   -78.4,      18.3},
{  -176.7,    10.5,   176.0,    14.9,   175.4,    16.2,    65.1,      17.5},
{  -177.2,     9.6,   177.1,    11.6,   179.5,    13.1,   179.9,      15.7},
{  -177.1,     8.9,   177.0,    13.0,  -176.6,    16.0,   -67.4,      17.7},
{   179.6,    14.4,  -178.0,    15.8,   -75.8,    20.6,    70.6,      21.7},
{  -175.0,    13.3,  -177.3,    16.0,   -70.2,    17.1,  -176.8,      17.9},
{  -175.4,    12.4,  -178.7,    18.2,   -71.0,    18.9,   -65.1,      18.5},
{  -162.4,    20.1,   -81.9,    20.4,    82.2,    20.8,    74.2,      20.7},
{  -163.0,    18.1,   -83.1,    19.9,    95.7,    19.6,  -178.9,      21.0},
{  -164.5,    19.9,   -82.8,    21.2,    88.3,    19.8,   -80.2,      19.2},
{  -161.2,    17.2,   -77.8,    23.7,  -177.5,    20.0,    66.4,      14.4},
{  -165.2,    19.3,   -87.2,    19.8,  -177.2,    19.8,  -178.9,      18.5},
{  -166.1,    22.6,   -83.0,    20.9,  -177.8,    18.1,   -67.3,      18.4},
{  -160.1,    20.9,   -90.5,    18.7,   -80.9,    23.3,    68.3,      26.6},
{  -166.8,    19.3,   -96.6,    12.9,   -69.4,    15.3,   178.4,      14.6},
{  -168.0,    17.8,   -93.8,    14.0,   -76.0,    17.4,   -65.7,      16.6},
{   -88.0,    19.5,    81.2,    19.8,    68.0,    13.0,    69.3,      17.1},
{   -80.9,    21.8,    81.3,    21.0,    66.9,    19.5,  -179.6,      24.1},
{   -86.8,    20.3,    86.0,    19.6,    84.3,    20.7,   -83.2,      18.3},
{   -80.7,    23.8,    75.7,    23.4,   176.2,    22.0,    62.3,      19.4},
{   -81.0,    23.1,    74.0,    21.2,  -178.3,    18.4,   177.9,      18.5},
{   -87.6,    17.8,    74.5,    25.3,   178.0,    24.9,   -61.8,      21.2},
{   -86.6,    21.7,    83.2,    24.1,   -85.6,    20.2,    68.2,      26.5},
{   -82.6,    18.5,    71.2,    28.5,   -96.3,    16.5,  -172.0,      20.3},
{   -85.8,    22.3,    78.0,    23.9,   -86.1,    21.0,   -66.3,      16.9},
{   -70.1,    12.8,   178.9,    16.4,    73.0,    18.8,    72.3,      18.0},
{   -68.5,    11.2,   176.2,    15.7,    69.4,    16.1,   177.3,      15.9},
{   -70.3,    15.4,   174.9,    19.8,    84.2,    17.2,   -76.5,      18.0},
{   -67.1,    10.4,  -178.4,    14.6,   177.7,    16.7,    66.5,      17.3},
{   -67.1,     9.0,  -179.4,    11.6,  -179.4,    13.4,   179.5,      15.0},
{   -66.8,     9.8,  -178.0,    12.9,  -177.5,    15.1,   -65.9,      16.9},
{   -66.8,    13.1,  -174.8,    18.2,   -76.5,    23.6,    78.2,      24.3},
{   -67.6,    10.3,  -175.0,    14.5,   -72.6,    15.3,  -176.5,      16.1},
{   -65.1,    11.1,  -174.8,    14.1,   -72.3,    16.3,   -65.7,      14.5},
{   -64.8,    10.7,   -75.8,    23.0,    85.4,    20.4,    76.5,      20.7},
{   -62.3,    13.4,   -76.6,    19.6,    91.0,    18.1,  -179.6,      19.6},
{   -62.3,    14.4,   -83.2,    20.5,    88.3,    19.6,   -84.3,      19.4},
{   -63.5,    13.1,   -67.5,    15.3,  -177.9,    18.8,    64.3,      19.8},
{   -62.2,    10.8,   -66.1,    12.4,  -177.3,    12.5,  -177.9,      14.8},
{   -60.1,    10.5,   -67.0,    13.2,  -174.3,    18.3,   -69.4,      18.3},
{   -65.2,    13.9,   -68.9,    18.8,   -76.9,    24.7,    78.3,      21.6},
{   -63.4,    12.1,   -66.5,    17.6,   -70.5,    14.3,  -179.8,      14.1},
{   -60.5,    11.4,   -68.5,    19.4,   -73.1,    16.7,   -72.7,      22.6},
{    60.8,    12.7,    78.3,     8.1,    73.3,    14.2,     0.0,     999.9},
{    57.6,    17.2,    78.1,    12.1,   177.7,    26.3,     0.0,     999.9},
{    59.4,    15.4,    77.8,    12.6,   -97.4,    20.6,     0.0,     999.9},
{    64.4,     8.4,  -178.3,    12.5,    76.1,    17.4,     0.0,     999.9},
{    64.9,     7.3,  -176.0,     7.7,  -175.0,    19.5,     0.0,     999.9},
{    63.4,     9.8,   179.1,    11.4,   -71.8,    14.7,     0.0,     999.9},
{    72.5,    13.4,   -70.1,    14.6,    99.6,    10.6,     0.0,     999.9},
{    72.2,    12.8,   -70.4,    14.7,   174.3,    23.8,     0.0,     999.9},
{    72.6,    13.3,   -72.0,    15.5,   -71.4,    11.6,     0.0,     999.9},
{  -176.0,     9.6,    64.0,     8.8,    72.5,    14.4,     0.0,     999.9},
{  -174.5,    11.1,    69.3,    10.4,  -166.1,    22.4,     0.0,     999.9},
{  -172.4,    12.9,    76.6,    14.3,   -96.9,    20.7,     0.0,     999.9},
{  -177.8,    10.7,   179.2,    12.3,    72.1,    14.1,     0.0,     999.9},
{  -177.4,    12.5,   174.6,    12.4,   177.3,    20.8,     0.0,     999.9},
{  -174.6,    12.3,   179.7,    10.3,   -71.6,    16.9,     0.0,     999.9},
{  -178.7,     9.5,   -73.7,    12.9,    99.6,    10.6,     0.0,     999.9},
{  -178.2,    13.5,   -78.4,    12.2,   173.0,    21.9,     0.0,     999.9},
{  -178.8,     7.3,   -82.3,     7.2,   -73.4,    11.7,     0.0,     999.9},
{   -78.6,    15.2,    67.3,    16.8,    73.0,    19.8,     0.0,     999.9},
{   -78.9,    13.8,    69.3,    13.8,  -175.6,    23.8,     0.0,     999.9},
{   -77.7,    13.5,    70.6,    14.5,   -98.9,    19.1,     0.0,     999.9},
{   -67.4,     9.4,   178.0,    11.6,    70.4,    14.2,     0.0,     999.9},
{   -67.3,     9.1,   179.2,     9.6,  -177.6,    19.6,     0.0,     999.9},
{   -67.7,     9.0,  -176.3,    11.9,   -73.7,    13.2,     0.0,     999.9},
{   -65.8,     8.2,   -64.9,    11.1,    99.6,    10.5,     0.0,     999.9},
{   -62.7,     8.2,   -64.0,    12.5,   171.9,    24.2,     0.0,     999.9},
{   -65.8,    10.3,   -61.8,    10.0,   -69.4,    13.4,     0.0,     999.9},
{    63.0,    10.0,    90.4,     9.6,     0.0,   999.9,     0.0,     999.9},
{    62.0,    12.3,    -6.5,    29.7,     0.0,   999.9,     0.0,     999.9},
{  -178.8,    10.4,    78.1,    13.6,     0.0,   999.9,     0.0,     999.9},
{  -174.2,     9.5,    21.2,    24.3,     0.0,   999.9,     0.0,     999.9},
{   -66.9,    10.6,    97.7,    16.2,     0.0,   999.9,     0.0,     999.9},
{   -68.2,    10.4,   -15.3,    22.0,     0.0,   999.9,     0.0,     999.9},
{    27.0,     8.8,   -34.6,    10.9,     0.0,   999.9,     0.0,     999.9},
{   -25.1,     8.1,    36.3,     9.5,     0.0,   999.9,     0.0,     999.9},
{    65.3,    10.5,     0.0,   999.9,     0.0,   999.9,     0.0,     999.9},
{   179.0,    11.6,     0.0,   999.9,     0.0,   999.9,     0.0,     999.9},
{   -63.9,    10.5,     0.0,   999.9,     0.0,   999.9,     0.0,     999.9},
{    61.1,     8.7,     0.0,   999.9,     0.0,   999.9,     0.0,     999.9},
{  -173.3,    10.6,     0.0,   999.9,     0.0,   999.9,     0.0,     999.9},
{   -60.4,     7.5,     0.0,   999.9,     0.0,   999.9,     0.0,     999.9},
{    61.2,    10.4,   -89.9,     9.8,     0.0,   999.9,     0.0,     999.9},
{    66.0,     8.2,    -6.3,    42.3,     0.0,   999.9,     0.0,     999.9},
{    61.5,    10.7,    89.2,    10.0,     0.0,   999.9,     0.0,     999.9},
{  -177.3,    12.5,  -104.9,    14.3,     0.0,   999.9,     0.0,     999.9},
{  -175.1,    12.4,    17.3,    30.8,     0.0,   999.9,     0.0,     999.9},
{  -179.8,     8.8,    84.7,     9.8,     0.0,   999.9,     0.0,     999.9},
{   -70.1,    13.6,   -92.2,    14.1,     0.0,   999.9,     0.0,     999.9},
{   -68.6,     9.3,    -4.2,    26.1,     0.0,   999.9,     0.0,     999.9},
{   -67.4,    10.9,   100.3,    15.7,     0.0,   999.9,     0.0,     999.9},
{    63.6,    10.9,    90.2,    11.3,     0.0,   999.9,     0.0,     999.9},
{    69.7,     8.4,    -9.5,    16.5,     0.0,   999.9,     0.0,     999.9},
{  -179.6,    10.6,    77.5,    13.7,     0.0,   999.9,     0.0,     999.9},
{  -174.2,    11.4,    21.0,    24.4,     0.0,   999.9,     0.0,     999.9},
{   -65.8,    10.9,    97.8,    16.3,     0.0,   999.9,     0.0,     999.9},
{   -67.2,    10.4,   -16.7,    22.8,     0.0,   999.9,     0.0,     999.9},
{    65.5,    11.1,     0.0,   999.9,     0.0,   999.9,     0.0,     999.9},
{   175.9,     6.9,     0.0,   999.9,     0.0,   999.9,     0.0,     999.9},
{   -61.7,     8.4,     0.0,   999.9,     0.0,   999.9,     0.0,     999.9}};

// Vector type lookup table, 20 blocks
static int Tab6[20][4] = {{17, 2,11,12},
			  { 7,11, 2, 4},
			  { 9,10, 4, 5},
			  {15, 2, 1,31},
			  {16, 2,12,32},
			  { 2,10,27,28},
			  {11,10,27,19},
			  { 3, 3,29,30},
			  {13, 3,29,22},
			  { 8, 6, 3,25},
			  {14,14,23,24},
			  { 1, 2, 1,34},
			  {10,35,10, 6},
			  { 4,10, 4, 5},
			  {19,10, 4, 5},
			  {18,10, 4, 5},
			  { 6,10,18, 5},
			  {12, 2, 1,10},
              {99, 0, 1,26},
              {99, 1, 2,17}};

// LCPO parameters lookup table, 158 atom types
// Journal of Computational Chemistry, Vol. 20, No. 2, 217-230 (1999)
static double Tab7[159][5] = {
{   0.000000000,   0.000000000,   0.000000000,   0.000000000,   0.000000000},
{   1.650000000,   0.411020000,  -0.122540000,  -0.000075448,   0.000118040},
{   1.700000000,   0.233480000,  -0.072627000,  -0.000200790,   0.000079670},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.600000000,   0.685630000,  -0.186800000,  -0.001355700,   0.000237430},
{   1.700000000,   0.778870000,  -0.280630000,  -0.001296800,   0.000393280},
{   1.650000000,   0.411020000,  -0.122540000,  -0.000075448,   0.000118040},
{   1.700000000,   0.233480000,  -0.072627000,  -0.000200790,   0.000079670},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.600000000,   0.685630000,  -0.186800000,  -0.001355700,   0.000237430},
{   1.700000000,   0.564820000,  -0.196080000,  -0.001021900,   0.000265800},
{   1.900000000,   0.772200000,  -0.263930000,   0.001062900,   0.000217900},
{   1.650000000,   0.411020000,  -0.122540000,  -0.000075448,   0.000118040},
{   1.700000000,   0.233480000,  -0.072627000,  -0.000200790,   0.000079670},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.600000000,   0.685630000,  -0.186800000,  -0.001355700,   0.000237430},
{   1.700000000,   0.564820000,  -0.196080000,  -0.001021900,   0.000265800},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.600000000,   0.888570000,  -0.334210000,  -0.001868300,   0.000493720},
{   1.650000000,   0.411020000,  -0.122540000,  -0.000075448,   0.000118040},
{   1.700000000,   0.233480000,  -0.072627000,  -0.000200790,   0.000079670},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.600000000,   0.685630000,  -0.186800000,  -0.001355700,   0.000237430},
{   1.700000000,   0.564820000,  -0.196080000,  -0.001021900,   0.000265800},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.700000000,   0.564820000,  -0.196080000,  -0.001021900,   0.000265800},
{   1.600000000,   0.888570000,  -0.334210000,  -0.001868300,   0.000493720},
{   1.650000000,   0.411020000,  -0.122540000,  -0.000075448,   0.000118040},
{   1.700000000,   0.233480000,  -0.072627000,  -0.000200790,   0.000079670},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.600000000,   0.685630000,  -0.186800000,  -0.001355700,   0.000237430},
{   1.700000000,   0.564820000,  -0.196080000,  -0.001021900,   0.000265800},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.700000000,   0.512450000,  -0.159660000,  -0.000197810,   0.000163920},
{   1.700000000,   0.512450000,  -0.159660000,  -0.000197810,   0.000163920},
{   1.700000000,   0.512450000,  -0.159660000,  -0.000197810,   0.000163920},
{   1.650000000,   0.411020000,  -0.122540000,  -0.000075448,   0.000118040},
{   1.700000000,   0.564820000,  -0.196080000,  -0.001021900,   0.000265800},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.600000000,   0.685630000,  -0.186800000,  -0.001355700,   0.000237430},
{   1.650000000,   0.411020000,  -0.122540000,  -0.000075448,   0.000118040},
{   1.700000000,   0.233480000,  -0.072627000,  -0.000200790,   0.000079670},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.600000000,   0.685630000,  -0.186800000,  -0.001355700,   0.000237430},
{   1.700000000,   0.564820000,  -0.196080000,  -0.001021900,   0.000265800},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.650000000,   0.411020000,  -0.122540000,  -0.000075448,   0.000118040},
{   1.700000000,   0.512450000,  -0.159660000,  -0.000197810,   0.000163920},
{   1.700000000,   0.512450000,  -0.159660000,  -0.000197810,   0.000163920},
{   1.650000000,   0.411020000,  -0.122540000,  -0.000075448,   0.000118040},
{   1.650000000,   0.411020000,  -0.122540000,  -0.000075448,   0.000118040},
{   1.700000000,   0.233480000,  -0.072627000,  -0.000200790,   0.000079670},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.600000000,   0.685630000,  -0.186800000,  -0.001355700,   0.000237430},
{   1.700000000,   0.233480000,  -0.072627000,  -0.000200790,   0.000079670},
{   1.700000000,   0.778870000,  -0.280630000,  -0.001296800,   0.000393280},
{   1.700000000,   0.564820000,  -0.196080000,  -0.001021900,   0.000265800},
{   1.700000000,   0.778870000,  -0.280630000,  -0.001296800,   0.000393280},
{   1.650000000,   0.411020000,  -0.122540000,  -0.000075448,   0.000118040},
{   1.700000000,   0.233480000,  -0.072627000,  -0.000200790,   0.000079670},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.600000000,   0.685630000,  -0.186800000,  -0.001355700,   0.000237430},
{   1.700000000,   0.564820000,  -0.196080000,  -0.001021900,   0.000265800},
{   1.700000000,   0.564820000,  -0.196080000,  -0.001021900,   0.000265800},
{   1.700000000,   0.564820000,  -0.196080000,  -0.001021900,   0.000265800},
{   1.700000000,   0.564820000,  -0.196080000,  -0.001021900,   0.000265800},
{   1.650000000,   0.078602000,  -0.291980000,  -0.000653700,   0.000362470},
{   1.650000000,   0.411020000,  -0.122540000,  -0.000075448,   0.000118040},
{   1.700000000,   0.233480000,  -0.072627000,  -0.000200790,   0.000079670},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.600000000,   0.685630000,  -0.186800000,  -0.001355700,   0.000237430},
{   1.700000000,   0.564820000,  -0.196080000,  -0.001021900,   0.000265800},
{   1.700000000,   0.233480000,  -0.072627000,  -0.000200790,   0.000079670},
{   1.700000000,   0.778870000,  -0.280630000,  -0.001296800,   0.000393280},
{   1.650000000,   0.411020000,  -0.122540000,  -0.000075448,   0.000118040},
{   1.700000000,   0.233480000,  -0.072627000,  -0.000200790,   0.000079670},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.600000000,   0.685630000,  -0.186800000,  -0.001355700,   0.000237430},
{   1.700000000,   0.564820000,  -0.196080000,  -0.001021900,   0.000265800},
{   1.700000000,   0.564820000,  -0.196080000,  -0.001021900,   0.000265800},
{   1.900000000,   0.545810000,  -0.194770000,  -0.001287300,   0.000292470},
{   1.700000000,   0.778870000,  -0.280630000,  -0.001296800,   0.000393280},
{   1.650000000,   0.411020000,  -0.122540000,  -0.000075448,   0.000118040},
{   1.700000000,   0.233480000,  -0.072627000,  -0.000200790,   0.000079670},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.600000000,   0.685630000,  -0.186800000,  -0.001355700,   0.000237430},
{   1.700000000,   0.564820000,  -0.196080000,  -0.001021900,   0.000265800},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.600000000,   0.685630000,  -0.186800000,  -0.001355700,   0.000237430},
{   1.650000000,   0.735110000,  -0.221160000,  -0.000891480,   0.000252300},
{   1.650000000,   0.062577000,  -0.017874000,  -0.000083120,   0.000019849},
{   1.700000000,   0.233480000,  -0.072627000,  -0.000200790,   0.000079670},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.600000000,   0.685630000,  -0.186800000,  -0.001355700,   0.000237430},
{   1.700000000,   0.564820000,  -0.196080000,  -0.001021900,   0.000265800},
{   1.700000000,   0.564820000,  -0.196080000,  -0.001021900,   0.000265800},
{   1.700000000,   0.564820000,  -0.196080000,  -0.001021900,   0.000265800},
{   1.650000000,   0.411020000,  -0.122540000,  -0.000075448,   0.000118040},
{   1.700000000,   0.233480000,  -0.072627000,  -0.000200790,   0.000079670},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.600000000,   0.685630000,  -0.186800000,  -0.001355700,   0.000237430},
{   1.700000000,   0.564820000,  -0.196080000,  -0.001021900,   0.000265800},
{   1.700000000,   0.564820000,  -0.196080000,  -0.001021900,   0.000265800},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.600000000,   0.685630000,  -0.186800000,  -0.001355700,   0.000237430},
{   1.650000000,   0.735110000,  -0.221160000,  -0.000891480,   0.000252300},
{   1.650000000,   0.411020000,  -0.122540000,  -0.000075448,   0.000118040},
{   1.700000000,   0.233480000,  -0.072627000,  -0.000200790,   0.000079670},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.600000000,   0.685630000,  -0.186800000,  -0.001355700,   0.000237430},
{   1.700000000,   0.564820000,  -0.196080000,  -0.001021900,   0.000265800},
{   1.700000000,   0.564820000,  -0.196080000,  -0.001021900,   0.000265800},
{   1.700000000,   0.564820000,  -0.196080000,  -0.001021900,   0.000265800},
{   1.650000000,   0.411020000,  -0.122540000,  -0.000075448,   0.000118040},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.650000000,   0.735110000,  -0.221160000,  -0.000891480,   0.000252300},
{   1.650000000,   0.411020000,  -0.122540000,  -0.000075448,   0.000118040},
{   1.700000000,   0.233480000,  -0.072627000,  -0.000200790,   0.000079670},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.600000000,   0.685630000,  -0.186800000,  -0.001355700,   0.000237430},
{   1.700000000,   0.564820000,  -0.196080000,  -0.001021900,   0.000265800},
{   1.600000000,   0.779140000,  -0.252620000,  -0.001605600,   0.000350710},
{   1.650000000,   0.411020000,  -0.122540000,  -0.000075448,   0.000118040},
{   1.700000000,   0.233480000,  -0.072627000,  -0.000200790,   0.000079670},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.600000000,   0.685630000,  -0.186800000,  -0.001355700,   0.000237430},
{   1.700000000,   0.233480000,  -0.072627000,  -0.000200790,   0.000079670},
{   1.600000000,   0.779140000,  -0.252620000,  -0.001605600,   0.000350710},
{   1.700000000,   0.778870000,  -0.280630000,  -0.001296800,   0.000393280},
{   1.650000000,   0.411020000,  -0.122540000,  -0.000075448,   0.000118040},
{   1.700000000,   0.233480000,  -0.072627000,  -0.000200790,   0.000079670},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.600000000,   0.685630000,  -0.186800000,  -0.001355700,   0.000237430},
{   1.700000000,   0.233480000,  -0.072627000,  -0.000200790,   0.000079670},
{   1.700000000,   0.778870000,  -0.280630000,  -0.001296800,   0.000393280},
{   1.650000000,   0.411020000,  -0.122540000,  -0.000075448,   0.000118040},
{   1.700000000,   0.233480000,  -0.072627000,  -0.000200790,   0.000079670},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.600000000,   0.685630000,  -0.186800000,  -0.001355700,   0.000237430},
{   1.700000000,   0.564820000,  -0.196080000,  -0.001021900,   0.000265800},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.700000000,   0.512450000,  -0.159660000,  -0.000197810,   0.000163920},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.650000000,   0.411020000,  -0.122540000,  -0.000075448,   0.000118040},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.700000000,   0.512450000,  -0.159660000,  -0.000197810,   0.000163920},
{   1.700000000,   0.512450000,  -0.159660000,  -0.000197810,   0.000163920},
{   1.700000000,   0.512450000,  -0.159660000,  -0.000197810,   0.000163920},
{   1.700000000,   0.512450000,  -0.159660000,  -0.000197810,   0.000163920},
{   1.650000000,   0.411020000,  -0.122540000,  -0.000075448,   0.000118040},
{   1.700000000,   0.233480000,  -0.072627000,  -0.000200790,   0.000079670},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.600000000,   0.685630000,  -0.186800000,  -0.001355700,   0.000237430},
{   1.700000000,   0.564820000,  -0.196080000,  -0.001021900,   0.000265800},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.700000000,   0.512450000,  -0.159660000,  -0.000197810,   0.000163920},
{   1.700000000,   0.512450000,  -0.159660000,  -0.000197810,   0.000163920},
{   1.700000000,   0.070344000,  -0.019015000,  -0.000022009,   0.000016875},
{   1.600000000,   0.493920000,  -0.160380000,  -0.000155120,   0.000164530}};

// Lookup table:
// Radii of Chothia (1975) and solvation parameter Lomize (2006)
static double RSigma[159][2] = {
{   0.000000000,   0.000000000},
{   1.650000000, -53.000000000},
{   1.870000000,  22.600000000},
{   1.760000000,  19.000000000},
{   1.400000000, -57.000000000},
{   1.870000000,  22.600000000},
{   1.650000000, -53.000000000},
{   1.870000000,  22.600000000},
{   1.760000000,  19.000000000},
{   1.400000000, -57.000000000},
{   1.870000000,  22.600000000},
{   1.850000000, -10.000000000},
{   1.650000000, -53.000000000},
{   1.870000000,  22.600000000},
{   1.760000000,  19.000000000},
{   1.400000000, -57.000000000},
{   1.870000000,  22.600000000},
{   1.760000000,  19.000000000},
{   1.400000000, -57.000000000},
{   1.650000000, -53.000000000},
{   1.870000000,  22.600000000},
{   1.760000000,  19.000000000},
{   1.400000000, -57.000000000},
{   1.870000000,  22.600000000},
{   1.870000000,  22.600000000},
{   1.760000000,  19.000000000},
{   1.400000000, -57.000000000},
{   1.650000000, -53.000000000},
{   1.870000000,  22.600000000},
{   1.760000000,  19.000000000},
{   1.400000000, -57.000000000},
{   1.870000000,  22.600000000},
{   1.760000000,  19.000000000},
{   1.760000000,  19.000000000},
{   1.760000000,  19.000000000},
{   1.760000000,  19.000000000},
{   1.650000000, -53.000000000},
{   1.870000000,  22.600000000},
{   1.760000000,  19.000000000},
{   1.400000000, -57.000000000},
{   1.650000000, -53.000000000},
{   1.870000000,  22.600000000},
{   1.760000000,  19.000000000},
{   1.400000000, -57.000000000},
{   1.870000000,  22.600000000},
{   1.760000000,  19.000000000},
{   1.650000000, -53.000000000},
{   1.760000000,  19.000000000},
{   1.760000000,  19.000000000},
{   1.650000000, -53.000000000},
{   1.650000000, -53.000000000},
{   1.870000000,  22.600000000},
{   1.760000000,  19.000000000},
{   1.400000000, -57.000000000},
{   1.870000000,  22.600000000},
{   1.870000000,  22.600000000},
{   1.870000000,  22.600000000},
{   1.870000000,  22.600000000},
{   1.650000000, -53.000000000},
{   1.870000000,  22.600000000},
{   1.760000000,  19.000000000},
{   1.400000000, -57.000000000},
{   1.870000000,  22.600000000},
{   1.870000000,  22.600000000},
{   1.870000000,  22.600000000},
{   1.870000000,  22.600000000},
{   1.500000000, -53.000000000},
{   1.650000000, -53.000000000},
{   1.870000000,  22.600000000},
{   1.760000000,  19.000000000},
{   1.400000000, -57.000000000},
{   1.870000000,  22.600000000},
{   1.870000000,  22.600000000},
{   1.870000000,  22.600000000},
{   1.650000000, -53.000000000},
{   1.870000000,  22.600000000},
{   1.760000000,  19.000000000},
{   1.400000000, -57.000000000},
{   1.870000000,  22.600000000},
{   1.870000000,  22.600000000},
{   1.850000000, -10.000000000},
{   1.870000000,  22.600000000},
{   1.650000000, -53.000000000},
{   1.870000000,  22.600000000},
{   1.760000000,  19.000000000},
{   1.400000000, -57.000000000},
{   1.870000000,  22.600000000},
{   1.760000000,  19.000000000},
{   1.400000000, -57.000000000},
{   1.650000000, -53.000000000},
{   1.650000000, -53.000000000},
{   1.870000000,  22.600000000},
{   1.760000000,  19.000000000},
{   1.400000000, -57.000000000},
{   1.870000000,  22.600000000},
{   1.870000000,  22.600000000},
{   1.870000000,  22.600000000},
{   1.650000000, -53.000000000},
{   1.870000000,  22.600000000},
{   1.760000000,  19.000000000},
{   1.400000000, -57.000000000},
{   1.870000000,  22.600000000},
{   1.870000000,  22.600000000},
{   1.760000000,  19.000000000},
{   1.400000000, -57.000000000},
{   1.650000000, -53.000000000},
{   1.650000000, -53.000000000},
{   1.870000000,  22.600000000},
{   1.760000000,  19.000000000},
{   1.400000000, -57.000000000},
{   1.870000000,  22.600000000},
{   1.870000000,  22.600000000},
{   1.870000000,  22.600000000},
{   1.650000000, -53.000000000},
{   1.760000000,  19.000000000},
{   1.650000000, -53.000000000},
{   1.650000000, -53.000000000},
{   1.870000000,  22.600000000},
{   1.760000000,  19.000000000},
{   1.400000000, -57.000000000},
{   1.870000000,  22.600000000},
{   1.400000000, -57.000000000},
{   1.650000000, -53.000000000},
{   1.870000000,  22.600000000},
{   1.760000000,  19.000000000},
{   1.400000000, -57.000000000},
{   1.870000000,  22.600000000},
{   1.400000000, -57.000000000},
{   1.870000000,  22.600000000},
{   1.650000000, -53.000000000},
{   1.870000000,  22.600000000},
{   1.760000000,  19.000000000},
{   1.400000000, -57.000000000},
{   1.870000000,  22.600000000},
{   1.870000000,  22.600000000},
{   1.650000000, -53.000000000},
{   1.870000000,  22.600000000},
{   1.760000000,  19.000000000},
{   1.400000000, -57.000000000},
{   1.870000000,  22.600000000},
{   1.760000000,  19.000000000},
{   1.760000000,  19.000000000},
{   1.760000000,  19.000000000},
{   1.650000000, -53.000000000},
{   1.760000000,  19.000000000},
{   1.760000000,  19.000000000},
{   1.760000000,  19.000000000},
{   1.760000000,  19.000000000},
{   1.760000000,  19.000000000},
{   1.650000000, -53.000000000},
{   1.870000000,  22.600000000},
{   1.760000000,  19.000000000},
{   1.400000000, -57.000000000},
{   1.870000000,  22.600000000},
{   1.760000000,  19.000000000},
{   1.760000000,  19.000000000},
{   1.760000000,  19.000000000},
{   1.760000000,  19.000000000},
{   1.400000000, -57.000000000}};

// lookup table of evenly distributed points on an unit sphere surface
static double UnitSphere[960][3] = {
        { 0.142065,  0.276550,  0.950441}, {-0.010435,  0.180640,  0.983494},
        { 0.093860,  0.192683,  0.976762}, { 0.148649,  0.117947,  0.981831},
        {-0.164403,  0.075573,  0.983494}, {-0.309310, -0.031464,  0.950441},
        {-0.213641, -0.017153,  0.976762}, {-0.164023, -0.095418,  0.981831},
        {-0.061830,  0.090608,  0.993965}, {-0.061717, -0.080548,  0.994838},
        {-0.008704,  0.012755,  0.999881}, { 0.097504,  0.028104,  0.994838},
        {-0.010177, -0.156740,  0.987587}, { 0.144419, -0.211636,  0.966620},
        { 0.095021, -0.139247,  0.985688}, { 0.149663, -0.047667,  0.987587},
        { 0.659784, -0.031464,  0.750796}, { 0.539741,  0.075573,  0.838432},
        { 0.582308, -0.017153,  0.812787}, { 0.538734, -0.095418,  0.837055},
        { 0.398310,  0.180640,  0.899288}, { 0.245163,  0.276550,  0.929201},
        { 0.299846,  0.192683,  0.934326}, { 0.251522,  0.117947,  0.960638},
        { 0.449659,  0.090608,  0.888593}, { 0.303643,  0.028104,  0.952371},
        { 0.403196,  0.012755,  0.915025}, { 0.449900, -0.080548,  0.889439},
        { 0.252866, -0.047667,  0.966326}, { 0.249395, -0.211636,  0.944993},
        { 0.302308, -0.139247,  0.942985}, { 0.399691, -0.156740,  0.903150},
        {-0.348009,  0.509982,  0.786643}, {-0.364005,  0.361769,  0.858268},
        {-0.306882,  0.449713,  0.838798}, {-0.204165,  0.470843,  0.858268},
        {-0.368013,  0.203516,  0.907275}, {-0.360619,  0.043725,  0.931688},
        {-0.316346,  0.133354,  0.939224}, {-0.215792,  0.150880,  0.964712},
        {-0.317683,  0.294553,  0.901286}, {-0.164646,  0.241277,  0.956387},
        {-0.214764,  0.314722,  0.924568}, {-0.158462,  0.403205,  0.901286},
        {-0.061824,  0.255946,  0.964712}, { 0.090756,  0.351740,  0.931688},
        {-0.008845,  0.343190,  0.939224}, {-0.055341,  0.416881,  0.907275},
        {-0.211240, -0.625845,  0.750796}, {-0.107557, -0.536442,  0.837055},
        {-0.196321, -0.548484,  0.812787}, {-0.267147, -0.475041,  0.838432},
        { 0.000236, -0.429325,  0.903150}, { 0.106147, -0.309387,  0.944993},
        { 0.019450, -0.332267,  0.942985}, {-0.047802, -0.252840,  0.966326},
        {-0.089017, -0.448301,  0.889439}, {-0.136870, -0.272499,  0.952371},
        {-0.158871, -0.370796,  0.915025}, {-0.248305, -0.385678,  0.888593},
        {-0.201528, -0.191210,  0.960638}, {-0.346897, -0.127467,  0.929201},
        {-0.288738, -0.208962,  0.934326}, {-0.313420, -0.305039,  0.899288},
        { 0.699503,  0.043725,  0.713291}, { 0.696646,  0.203516,  0.687943},
        { 0.661814,  0.133354,  0.737712}, { 0.579521,  0.150880,  0.800868},
        { 0.673594,  0.361769,  0.644511}, { 0.630591,  0.509983,  0.585041},
        { 0.613427,  0.449713,  0.649204}, { 0.526769,  0.470843,  0.707687},
        { 0.648047,  0.294553,  0.702334}, { 0.501791,  0.403205,  0.765266},
        { 0.562711,  0.314722,  0.764400}, { 0.529250,  0.241277,  0.813437},
        { 0.409434,  0.416881,  0.811526}, { 0.284882,  0.351740,  0.891696},
        { 0.379351,  0.343190,  0.859252}, { 0.438090,  0.255946,  0.861724},
        { 0.276003, -0.309387,  0.910001}, { 0.356751, -0.429325,  0.829704},
        { 0.354847, -0.332267,  0.873889}, { 0.425849, -0.252840,  0.868749},
        { 0.429643, -0.536442,  0.726386}, { 0.490790, -0.625845,  0.606170},
        { 0.501588, -0.548484,  0.669010}, { 0.576783, -0.475041,  0.664573},
        { 0.433318, -0.448301,  0.781832}, { 0.579301, -0.385678,  0.718097},
        { 0.507597, -0.370796,  0.777725}, { 0.502148, -0.272499,  0.820727},
        { 0.643341, -0.305039,  0.702184}, { 0.685915, -0.127466,  0.716431},
        { 0.634517, -0.208961,  0.744126}, { 0.564808, -0.191210,  0.802764},
        {-0.445452,  0.498942,  0.743391}, {-0.584618,  0.427737,  0.689393},
        {-0.499292,  0.427913,  0.753391}, {-0.459801,  0.350916,  0.815746},
        {-0.707664,  0.342972,  0.617723}, {-0.808789,  0.249821,  0.532399},
        {-0.747671,  0.259481,  0.611275}, {-0.715434,  0.181892,  0.674589},
        {-0.633362,  0.349398,  0.690488}, {-0.639427,  0.187484,  0.745642},
        {-0.597104,  0.271403,  0.754856}, {-0.509028,  0.272874,  0.816352},
        {-0.595202,  0.107892,  0.796300}, {-0.456318,  0.032883,  0.889209},
        {-0.507548,  0.111690,  0.854353}, {-0.463503,  0.192697,  0.864889},
        { 0.135742,  0.436900,  0.889209}, { 0.116528,  0.593572,  0.796300},
        { 0.081035,  0.513335,  0.854354}, {-0.010454,  0.501854,  0.864889},
        { 0.091454,  0.732507,  0.674589}, { 0.062235,  0.844202,  0.532399},
        { 0.030958,  0.790812,  0.611275}, {-0.061373,  0.783997,  0.617723},
        { 0.058537,  0.663770,  0.745643}, {-0.094445,  0.717151,  0.690488},
        {-0.035036,  0.654954,  0.754856}, {-0.068516,  0.573476,  0.816352},
        {-0.185164,  0.700322,  0.689393}, {-0.302204,  0.596693,  0.743391},
        {-0.216434,  0.620933,  0.753391}, {-0.159133,  0.556089,  0.815746},
        {-0.131372, -0.685178,  0.716431}, { 0.049500, -0.710273,  0.702184},
        {-0.036748, -0.667028,  0.744125}, {-0.027863, -0.595645,  0.802764},
        { 0.232067, -0.710273,  0.664573}, { 0.403843, -0.685178,  0.606170},
        { 0.327870, -0.667028,  0.669010}, { 0.342885, -0.595645,  0.726386},
        { 0.147936, -0.680038,  0.718097}, { 0.259470, -0.566933,  0.781832},
        { 0.160220, -0.607844,  0.777725}, { 0.070675, -0.566933,  0.820726},
        { 0.269715, -0.488718,  0.829704}, { 0.187471, -0.369801,  0.910001},
        { 0.180031, -0.451560,  0.873889}, { 0.080186, -0.488718,  0.868749},
        {-0.431392, -0.137040,  0.891696}, {-0.537458, -0.229271,  0.811526},
        {-0.457871, -0.228124,  0.859251}, {-0.398047, -0.314627,  0.861724},
        {-0.630483, -0.318858,  0.707687}, {-0.704780, -0.401265,  0.585040},
        {-0.642400, -0.407254,  0.649204}, {-0.582445, -0.495343,  0.644511},
        {-0.558394, -0.320257,  0.765266}, {-0.510541, -0.496059,  0.702334},
        {-0.498210, -0.409243,  0.764399}, {-0.417621, -0.404861,  0.813437},
        {-0.443487, -0.574503,  0.687943}, {-0.295735, -0.635419,  0.713291},
        {-0.365455, -0.567647,  0.737712}, {-0.351773, -0.484629,  0.800869},
        { 0.953363,  0.249821,  0.169376}, { 0.894196,  0.342972,  0.287722},
        { 0.928397,  0.259481,  0.265986}, { 0.923809,  0.181892,  0.336886},
        { 0.809496,  0.427738,  0.402190}, { 0.703004,  0.498942,  0.506796},
        { 0.756413,  0.427913,  0.494702}, { 0.744783,  0.350916,  0.567588},
        { 0.854704,  0.349398,  0.383930}, { 0.790242,  0.272874,  0.548687},
        { 0.846840,  0.271403,  0.457387}, { 0.882075,  0.187484,  0.432196},
        { 0.767608,  0.192697,  0.611266}, { 0.770620,  0.032883,  0.636446},
        { 0.803902,  0.111690,  0.584180}, { 0.861473,  0.107892,  0.496208},
        { 0.153263,  0.844202,  0.513647}, { 0.182623,  0.732507,  0.655807},
        { 0.213168,  0.790812,  0.573738}, { 0.300529,  0.783997,  0.543167},
        { 0.207697,  0.593572,  0.777518}, { 0.226770,  0.436900,  0.870456},
        { 0.263245,  0.513335,  0.816816}, { 0.351449,  0.501854,  0.790333},
        { 0.240944,  0.663770,  0.708065}, { 0.385598,  0.573477,  0.722799},
        { 0.330539,  0.654954,  0.679543}, { 0.359669,  0.717152,  0.596935},
        { 0.468598,  0.556089,  0.686426}, { 0.571420,  0.596694,  0.563414},
        { 0.496587,  0.620933,  0.606501}, { 0.442567,  0.700323,  0.560073},
        { 0.878628, -0.401265,  0.258841}, { 0.858857, -0.318858,  0.400866},
        { 0.846689, -0.407254,  0.342435}, { 0.789760, -0.495343,  0.361821},
        { 0.814449, -0.229271,  0.533018}, { 0.748706, -0.137040,  0.648582},
        { 0.760206, -0.228124,  0.608314}, { 0.706230, -0.314627,  0.634231},
        { 0.815396, -0.320257,  0.482250}, { 0.705126, -0.404861,  0.582139},
        { 0.759771, -0.409243,  0.505241}, { 0.746566, -0.496059,  0.443356},
        { 0.639671, -0.484629,  0.596620}, { 0.553582, -0.635419,  0.538322},
        { 0.627277, -0.567647,  0.533199}, { 0.679284, -0.574503,  0.456640},
        {-0.589414,  0.770371,  0.243147}, {-0.691910,  0.643752,  0.326871},
        {-0.616865,  0.717414,  0.323721}, {-0.560020,  0.727500,  0.396384},
        {-0.771571,  0.492950,  0.402092}, {-0.822949,  0.328280,  0.463667},
        {-0.775962,  0.416237,  0.473952}, {-0.721793,  0.421259,  0.549141},
        {-0.709675,  0.576731,  0.404650}, {-0.661673,  0.506272,  0.553062},
        {-0.653675,  0.584862,  0.480256}, {-0.579294,  0.662218,  0.475274},
        {-0.598792,  0.506276,  0.620591}, {-0.459869,  0.578830,  0.673406},
        {-0.527761,  0.585660,  0.615200}, {-0.516093,  0.662828,  0.542501},
        {-0.763951, -0.314554,  0.563414}, {-0.688655, -0.233611,  0.686426},
        {-0.759240, -0.236034,  0.606500}, {-0.813471, -0.156790,  0.560073},
        {-0.595443, -0.144298,  0.790333}, {-0.489504, -0.051880,  0.870456},
        {-0.573977, -0.057979,  0.816816}, {-0.628440,  0.022998,  0.777518},
        {-0.674586, -0.149986,  0.722799}, {-0.705928,  0.017632,  0.708064},
        {-0.730382, -0.069011,  0.679543}, {-0.798919, -0.073461,  0.596935},
        {-0.748672,  0.096998,  0.655807}, {-0.841976,  0.165058,  0.513646},
        {-0.814100,  0.089811,  0.573738}, {-0.839603,  0.005978,  0.543167},
        {-0.502466,  0.829703,  0.243147}, {-0.473263,  0.786703,  0.396384},
        {-0.443147,  0.835958,  0.323721}, {-0.347195,  0.878983,  0.326871},
        {-0.429056,  0.722221,  0.542501}, {-0.371337,  0.639244,  0.673406},
        {-0.352945,  0.704953,  0.615200}, {-0.253129,  0.742154,  0.620591},
        {-0.405447,  0.780851,  0.475274}, {-0.230200,  0.800706,  0.553062},
        {-0.306298,  0.821910,  0.480256}, {-0.278310,  0.871092,  0.404650},
        {-0.129121,  0.825694,  0.549142}, {-0.005662,  0.885991,  0.463667},
        {-0.104696,  0.874305,  0.473952}, {-0.177730,  0.898184,  0.402092},
        { 0.053324, -0.964447,  0.258841}, { 0.173326, -0.915993,  0.361821},
        { 0.070546, -0.936889,  0.342435}, {-0.016203, -0.915993,  0.400866},
        { 0.287315, -0.841980,  0.456640}, { 0.389865, -0.747138,  0.538322},
        { 0.299891, -0.791053,  0.533199}, { 0.218068, -0.772328,  0.596620},
        { 0.189740, -0.876033,  0.443356}, { 0.119927, -0.804197,  0.582139},
        { 0.104085, -0.856678,  0.505241}, { 0.000945, -0.876033,  0.482250},
        { 0.035501, -0.772328,  0.634231}, {-0.145350, -0.747137,  0.648582},
        {-0.064727, -0.791053,  0.608314}, {-0.083434, -0.841980,  0.533018},
        {-0.580197, -0.796670,  0.169376}, {-0.506169, -0.793915,  0.336886},
        {-0.580090, -0.769900,  0.265986}, {-0.645367, -0.707614,  0.287722},
        {-0.414536, -0.762848,  0.496208}, {-0.311566, -0.705594,  0.636446},
        {-0.397084, -0.707854,  0.584180}, {-0.459283, -0.644525,  0.611266},
        {-0.496161, -0.753015,  0.432196}, {-0.542194, -0.636371,  0.548687},
        {-0.561458, -0.689610,  0.457387}, {-0.636953, -0.668498,  0.383929},
        {-0.598292, -0.565589,  0.567588}, {-0.720899, -0.472719,  0.506796},
        {-0.674230, -0.548348,  0.494701}, {-0.693419, -0.597841,  0.402189},
        { 0.939203,  0.328280,  0.100644}, { 0.867672,  0.492950,  0.064390},
        { 0.900107,  0.416238,  0.128663}, { 0.880067,  0.421259,  0.219140},
        { 0.764766,  0.643752,  0.026779}, { 0.637523,  0.770371, -0.009616},
        { 0.694586,  0.717414,  0.053548}, { 0.671090,  0.727501,  0.142762},
        { 0.811827,  0.576732,  0.091203}, { 0.719976,  0.662218,  0.207610},
        { 0.790269,  0.584862,  0.182787}, { 0.826393,  0.506272,  0.246504},
        { 0.688492,  0.662828,  0.294343}, { 0.688586,  0.578830,  0.436812},
        { 0.727944,  0.585660,  0.356510}, { 0.795322,  0.506276,  0.333388},
        { 0.793692, -0.051880,  0.606104}, { 0.859336, -0.144298,  0.490632},
        { 0.850086, -0.057979,  0.523443}, { 0.884581,  0.022998,  0.465819},
        { 0.903889, -0.233611,  0.358344}, { 0.924434, -0.314554,  0.215588},
        { 0.937136, -0.236034,  0.257028}, { 0.968602, -0.156790,  0.192946},
        { 0.905342, -0.149985,  0.397316}, { 0.969804, -0.073461,  0.232558},
        { 0.939499, -0.069011,  0.335529}, { 0.928308,  0.017632,  0.371394},
        { 0.985924,  0.005978,  0.167088}, { 0.976435,  0.165058,  0.139033},
        { 0.974580,  0.089811,  0.205249}, { 0.946917,  0.096998,  0.306496},
        { 0.607263,  0.639244,  0.471804}, { 0.608542,  0.722222,  0.328744},
        { 0.567363,  0.704953,  0.425606}, { 0.477805,  0.742154,  0.470010},
        { 0.591397,  0.786703,  0.177053}, { 0.557655,  0.829704,  0.024750},
        { 0.535013,  0.835958,  0.122209}, { 0.448119,  0.878983,  0.163028},
        { 0.560284,  0.780851,  0.276323}, { 0.415585,  0.871092,  0.261700},
        { 0.471177,  0.821910,  0.320087}, { 0.430052,  0.800706,  0.417043},
        { 0.322184,  0.898184,  0.299105}, { 0.188464,  0.885991,  0.423675},
        { 0.283499,  0.874305,  0.393979}, { 0.335654,  0.825695,  0.453393},
        { 0.537751, -0.705594,  0.461477}, { 0.576908, -0.762848,  0.291960},
        { 0.595647, -0.707854,  0.379666}, { 0.663487, -0.644525,  0.379963},
        { 0.598108, -0.793915,  0.109393}, { 0.599900, -0.796670, -0.073737},
        { 0.637987, -0.769900,  0.015049}, { 0.706539, -0.707614,  0.009214},
        { 0.626586, -0.753014,  0.200898}, { 0.736836, -0.668498,  0.100914},
        { 0.696523, -0.689610,  0.198229}, { 0.714913, -0.636371,  0.289709},
        { 0.795922, -0.597841,  0.095369}, { 0.862509, -0.472719,  0.180596},
        { 0.814860, -0.548347,  0.187932}, { 0.773913, -0.565589,  0.284898},
        {-0.635102,  0.754881,  0.163708}, {-0.741664,  0.665917,  0.080560},
        {-0.708318,  0.686408,  0.164710}, {-0.737669,  0.628238,  0.247309},
        {-0.831159,  0.556009, -0.005306}, {-0.898352,  0.430169, -0.088985},
        {-0.885636,  0.464328, -0.007038}, {-0.913858,  0.399457,  0.072784},
        {-0.807220,  0.584943,  0.078979}, {-0.889599,  0.428997,  0.156767},
        {-0.837161,  0.522654,  0.161227}, {-0.801228,  0.545692,  0.245467},
        {-0.903436,  0.359676,  0.233317}, {-0.868637,  0.312790,  0.384229},
        {-0.867415,  0.385231,  0.314941}, {-0.817330,  0.477436,  0.322531},
        {-0.893722,  0.147515,  0.423675}, {-0.953826,  0.027443,  0.299105},
        {-0.917487,  0.054759,  0.393979}, {-0.891236, -0.011527,  0.453392},
        {-0.981859, -0.096824,  0.163028}, {-0.975905, -0.216788,  0.024750},
        {-0.973474, -0.193423,  0.122209}, {-0.948166, -0.263883,  0.177053},
        {-0.962650, -0.069407,  0.261700}, {-0.931373, -0.237045,  0.276323},
        {-0.937120, -0.139103,  0.320087}, {-0.902383, -0.108539,  0.417042},
        {-0.894372, -0.303357,  0.328744}, {-0.816640, -0.332417,  0.471803},
        {-0.863280, -0.271307,  0.425606}, {-0.865270, -0.174351,  0.470010},
        { 0.025419,  0.922888,  0.384229}, {-0.005554,  0.972385,  0.233317},
        {-0.042483,  0.948160,  0.314941}, {-0.146601,  0.935137,  0.322531},
        {-0.038797,  0.996593,  0.072784}, {-0.073048,  0.993351, -0.088985},
        {-0.109492,  0.993963, -0.007038}, {-0.214725,  0.976660, -0.005306},
        {-0.075148,  0.984772,  0.156767}, {-0.250395,  0.964917,  0.078979},
        {-0.181476,  0.970089,  0.161227}, {-0.216030,  0.945028,  0.245467},
        {-0.349695,  0.933393,  0.080560}, {-0.471386,  0.866600,  0.163708},
        {-0.380933,  0.909814,  0.164710}, {-0.316066,  0.915937,  0.247309},
        { 0.125737, -0.975487,  0.180596}, { 0.266524, -0.959098,  0.095369},
        { 0.213532, -0.958689,  0.187932}, { 0.244515, -0.926847,  0.284898},
        { 0.401328, -0.915888,  0.009214}, { 0.523132, -0.849056, -0.073737},
        { 0.484320, -0.874762,  0.015049}, { 0.521220, -0.846382,  0.109393},
        { 0.353858, -0.929839,  0.100914}, { 0.472752, -0.857989,  0.200898},
        { 0.388214, -0.899997,  0.198229}, { 0.331935, -0.897712,  0.289709},
        { 0.500020, -0.815316,  0.291960}, { 0.460982, -0.757980,  0.461477},
        { 0.441980, -0.812716,  0.379666}, { 0.358276, -0.852799,  0.379963},
        {-0.241049, -0.757980,  0.606104}, {-0.343909, -0.815316,  0.465819},
        {-0.255929, -0.812716,  0.523443}, {-0.178924, -0.852799,  0.490632},
        {-0.435542, -0.846382,  0.306496}, {-0.509680, -0.849056,  0.139033},
        {-0.438935, -0.874762,  0.205249}, {-0.365007, -0.915888,  0.167088},
        {-0.354854, -0.857989,  0.371394}, {-0.285159, -0.929839,  0.232558},
        {-0.278254, -0.899997,  0.335529}, {-0.190400, -0.897712,  0.397316},
        {-0.207127, -0.959099,  0.192946}, {-0.044118, -0.975487,  0.215588},
        {-0.121865, -0.958690,  0.257028}, {-0.112000, -0.926847,  0.358344},
        {-0.790032, -0.430169,  0.436811}, {-0.868215, -0.399457,  0.294343},
        {-0.810741, -0.464328,  0.356510}, {-0.761384, -0.556010,  0.333388},
        {-0.922091, -0.359676,  0.142762}, {-0.949774, -0.312790, -0.009616},
        {-0.921265, -0.385231,  0.053548}, {-0.878258, -0.477436,  0.026779},
        {-0.879125, -0.428997,  0.207610}, {-0.833008, -0.545692,  0.091203},
        {-0.832720, -0.522654,  0.182787}, {-0.772708, -0.584943,  0.246504},
        {-0.775352, -0.628238,  0.064390}, {-0.648094, -0.754881,  0.100644},
        {-0.715744, -0.686408,  0.128662}, {-0.713115, -0.665917,  0.219140},
        { 0.790032,  0.430169, -0.436811}, { 0.761384,  0.556010, -0.333388},
        { 0.810741,  0.464328, -0.356510}, { 0.868215,  0.399457, -0.294343},
        { 0.713115,  0.665917, -0.219140}, { 0.648094,  0.754881, -0.100644},
        { 0.715744,  0.686408, -0.128662}, { 0.775352,  0.628238, -0.064390},
        { 0.772708,  0.584943, -0.246504}, { 0.833008,  0.545692, -0.091203},
        { 0.832720,  0.522654, -0.182787}, { 0.879125,  0.428997, -0.207610},
        { 0.878258,  0.477436, -0.026779}, { 0.949774,  0.312790,  0.009616},
        { 0.921265,  0.385231, -0.053548}, { 0.922091,  0.359676, -0.142762},
        { 0.906224, -0.216788, -0.362990}, { 0.966347, -0.096824, -0.238325},
        { 0.942512, -0.193423, -0.272506}, { 0.940940, -0.263883, -0.212124},
        { 0.994380,  0.027443, -0.102248}, { 0.988407,  0.147515,  0.035935},
        { 0.998499,  0.054760, -0.000736}, { 0.997870, -0.011527,  0.064215},
        { 0.987702, -0.069407, -0.140095}, { 0.993741, -0.108539,  0.026419},
        { 0.987329, -0.139103, -0.076371}, { 0.964751, -0.237045, -0.114300},
        { 0.980585, -0.174351,  0.089743}, { 0.936624, -0.332417,  0.110612},
        { 0.961207, -0.271307,  0.049741}, { 0.951483, -0.303357, -0.051523},
        { 0.031928,  0.993351, -0.110612}, { 0.064406,  0.996593,  0.051523},
        { 0.097795,  0.993963, -0.049741}, { 0.195143,  0.976660, -0.089743},
        { 0.097319,  0.972385,  0.212125}, { 0.128517,  0.922888,  0.362990},
        { 0.163503,  0.948160,  0.272506}, { 0.262143,  0.935137,  0.238325},
        { 0.130991,  0.984773,  0.114300}, { 0.295459,  0.945028,  0.140095},
        { 0.230424,  0.970089,  0.076371}, { 0.261222,  0.964917, -0.026419},
        { 0.388078,  0.915937,  0.102248}, { 0.497708,  0.866600, -0.035935},
        { 0.415016,  0.909814,  0.000736}, { 0.353062,  0.933394, -0.064215},
        { 0.868637, -0.312790, -0.384229}, { 0.903436, -0.359676, -0.233317},
        { 0.867415, -0.385231, -0.314941}, { 0.817330, -0.477436, -0.322531},
        { 0.913858, -0.399457, -0.072784}, { 0.898352, -0.430169,  0.088985},
        { 0.885636, -0.464328,  0.007038}, { 0.831159, -0.556009,  0.005306},
        { 0.889599, -0.428997, -0.156767}, { 0.807220, -0.584943, -0.078979},
        { 0.837161, -0.522654, -0.161227}, { 0.801228, -0.545692, -0.245467},
        { 0.741664, -0.665917, -0.080560}, { 0.635102, -0.754881, -0.163708},
        { 0.708318, -0.686408, -0.164710}, { 0.737669, -0.628238, -0.247309},
        {-0.537751,  0.705594, -0.461477}, {-0.663487,  0.644525, -0.379963},
        {-0.595647,  0.707854, -0.379666}, {-0.576908,  0.762848, -0.291960},
        {-0.773913,  0.565589, -0.284898}, {-0.862509,  0.472719, -0.180596},
        {-0.814860,  0.548347, -0.187932}, {-0.795922,  0.597841, -0.095369},
        {-0.714913,  0.636371, -0.289709}, {-0.736836,  0.668498, -0.100914},
        {-0.696523,  0.689610, -0.198229}, {-0.626586,  0.753014, -0.200898},
        {-0.706539,  0.707614, -0.009214}, {-0.599900,  0.796670,  0.073737},
        {-0.637987,  0.769900, -0.015049}, {-0.598108,  0.793915, -0.109393},
        {-0.988407, -0.147515, -0.035935}, {-0.994380, -0.027443,  0.102248},
        {-0.998499, -0.054760,  0.000736}, {-0.997870,  0.011527, -0.064215},
        {-0.966347,  0.096824,  0.238325}, {-0.906224,  0.216788,  0.362990},
        {-0.942512,  0.193423,  0.272506}, {-0.940940,  0.263883,  0.212124},
        {-0.987702,  0.069407,  0.140095}, {-0.964751,  0.237045,  0.114300},
        {-0.987329,  0.139103,  0.076371}, {-0.993741,  0.108539, -0.026419},
        {-0.951483,  0.303357,  0.051523}, {-0.936624,  0.332417, -0.110612},
        {-0.961207,  0.271307, -0.049741}, {-0.980585,  0.174351, -0.089743},
        {-0.460982,  0.757980, -0.461477}, {-0.500020,  0.815316, -0.291960},
        {-0.441980,  0.812716, -0.379666}, {-0.358276,  0.852799, -0.379963},
        {-0.521220,  0.846382, -0.109393}, {-0.523132,  0.849056,  0.073737},
        {-0.484320,  0.874762, -0.015049}, {-0.401328,  0.915888, -0.009214},
        {-0.472752,  0.857989, -0.200898}, {-0.353858,  0.929839, -0.100914},
        {-0.388214,  0.899997, -0.198229}, {-0.331935,  0.897712, -0.289709},
        {-0.266524,  0.959098, -0.095369}, {-0.125737,  0.975487, -0.180596},
        {-0.213532,  0.958689, -0.187932}, {-0.244515,  0.926847, -0.284898},
        {-0.025419, -0.922888, -0.384229}, { 0.146601, -0.935137, -0.322531},
        { 0.042483, -0.948160, -0.314941}, { 0.005554, -0.972385, -0.233317},
        { 0.316066, -0.915937, -0.247309}, { 0.471386, -0.866600, -0.163708},
        { 0.380933, -0.909814, -0.164710}, { 0.349695, -0.933393, -0.080560},
        { 0.216030, -0.945028, -0.245467}, { 0.250395, -0.964917, -0.078979},
        { 0.181476, -0.970089, -0.161227}, { 0.075148, -0.984772, -0.156767},
        { 0.214725, -0.976660,  0.005306}, { 0.073048, -0.993351,  0.088985},
        { 0.109492, -0.993963,  0.007038}, { 0.038797, -0.996593, -0.072784},
        {-0.128517, -0.922888, -0.362990}, {-0.097319, -0.972385, -0.212125},
        {-0.163503, -0.948160, -0.272506}, {-0.262143, -0.935137, -0.238325},
        {-0.064406, -0.996593, -0.051523}, {-0.031928, -0.993351,  0.110612},
        {-0.097795, -0.993963,  0.049741}, {-0.195143, -0.976660,  0.089743},
        {-0.130991, -0.984773, -0.114300}, {-0.261222, -0.964917,  0.026419},
        {-0.230424, -0.970089, -0.076371}, {-0.295459, -0.945028, -0.140095},
        {-0.353062, -0.933394,  0.064215}, {-0.497708, -0.866600,  0.035935},
        {-0.415016, -0.909814, -0.000736}, {-0.388078, -0.915937, -0.102248},
        {-0.688586, -0.578830, -0.436812}, {-0.688492, -0.662828, -0.294343},
        {-0.727944, -0.585660, -0.356510}, {-0.795322, -0.506276, -0.333388},
        {-0.671090, -0.727501, -0.142762}, {-0.637523, -0.770371,  0.009616},
        {-0.694586, -0.717414, -0.053548}, {-0.764766, -0.643752, -0.026779},
        {-0.719976, -0.662218, -0.207610}, {-0.811827, -0.576732, -0.091203},
        {-0.790269, -0.584862, -0.182787}, {-0.826393, -0.506272, -0.246504},
        {-0.867672, -0.492950, -0.064390}, {-0.939203, -0.328280, -0.100644},
        {-0.900107, -0.416238, -0.128663}, {-0.880067, -0.421259, -0.219140},
        { 0.720899,  0.472719, -0.506796}, { 0.598292,  0.565589, -0.567588},
        { 0.674230,  0.548348, -0.494701}, { 0.693419,  0.597841, -0.402189},
        { 0.459283,  0.644525, -0.611266}, { 0.311566,  0.705594, -0.636446},
        { 0.397084,  0.707854, -0.584180}, { 0.414536,  0.762848, -0.496208},
        { 0.542194,  0.636371, -0.548687}, { 0.496161,  0.753015, -0.432196},
        { 0.561458,  0.689610, -0.457387}, { 0.636953,  0.668498, -0.383929},
        { 0.506169,  0.793915, -0.336886}, { 0.580197,  0.796670, -0.169376},
        { 0.580090,  0.769900, -0.265986}, { 0.645367,  0.707614, -0.287722},
        { 0.975905,  0.216788, -0.024750}, { 0.981859,  0.096824, -0.163028},
        { 0.973474,  0.193423, -0.122209}, { 0.948166,  0.263883, -0.177053},
        { 0.953826, -0.027443, -0.299105}, { 0.893722, -0.147515, -0.423675},
        { 0.917487, -0.054759, -0.393979}, { 0.891236,  0.011527, -0.453392},
        { 0.962650,  0.069407, -0.261700}, { 0.902383,  0.108539, -0.417042},
        { 0.937120,  0.139103, -0.320087}, { 0.931373,  0.237045, -0.276323},
        { 0.865270,  0.174351, -0.470010}, { 0.816640,  0.332417, -0.471803},
        { 0.863280,  0.271307, -0.425606}, { 0.894372,  0.303357, -0.328744},
        { 0.509680,  0.849056, -0.139033}, { 0.435542,  0.846382, -0.306496},
        { 0.438935,  0.874762, -0.205249}, { 0.365007,  0.915888, -0.167088},
        { 0.343909,  0.815316, -0.465819}, { 0.241049,  0.757980, -0.606104},
        { 0.255929,  0.812716, -0.523443}, { 0.178924,  0.852799, -0.490632},
        { 0.354854,  0.857989, -0.371394}, { 0.190400,  0.897712, -0.397316},
        { 0.278254,  0.899997, -0.335529}, { 0.285159,  0.929839, -0.232558},
        { 0.112000,  0.926847, -0.358344}, { 0.044118,  0.975487, -0.215588},
        { 0.121865,  0.958690, -0.257028}, { 0.207127,  0.959099, -0.192946},
        { 0.589414, -0.770371, -0.243147}, { 0.560020, -0.727500, -0.396384},
        { 0.616865, -0.717414, -0.323721}, { 0.691910, -0.643752, -0.326871},
        { 0.516093, -0.662828, -0.542501}, { 0.459869, -0.578830, -0.673406},
        { 0.527761, -0.585660, -0.615200}, { 0.598792, -0.506276, -0.620591},
        { 0.579294, -0.662218, -0.475274}, { 0.661673, -0.506272, -0.553062},
        { 0.653675, -0.584862, -0.480256}, { 0.709675, -0.576731, -0.404650},
        { 0.721793, -0.421259, -0.549141}, { 0.822949, -0.328280, -0.463667},
        { 0.775962, -0.416237, -0.473952}, { 0.771571, -0.492950, -0.402092},
        {-0.553582,  0.635419, -0.538322}, {-0.639671,  0.484629, -0.596620},
        {-0.627277,  0.567647, -0.533199}, {-0.679284,  0.574503, -0.456640},
        {-0.706230,  0.314627, -0.634231}, {-0.748706,  0.137040, -0.648582},
        {-0.760206,  0.228124, -0.608314}, {-0.814449,  0.229271, -0.533018},
        {-0.705126,  0.404861, -0.582139}, {-0.815396,  0.320257, -0.482250},
        {-0.759771,  0.409243, -0.505241}, {-0.746566,  0.496059, -0.443356},
        {-0.858857,  0.318858, -0.400866}, {-0.878628,  0.401265, -0.258841},
        {-0.846689,  0.407254, -0.342435}, {-0.789760,  0.495343, -0.361821},
        {-0.924434,  0.314554, -0.215588}, {-0.903889,  0.233611, -0.358344},
        {-0.937136,  0.236034, -0.257028}, {-0.968602,  0.156790, -0.192946},
        {-0.859336,  0.144298, -0.490632}, {-0.793692,  0.051880, -0.606104},
        {-0.850086,  0.057979, -0.523443}, {-0.884581, -0.022998, -0.465819},
        {-0.905342,  0.149985, -0.397316}, {-0.928308, -0.017632, -0.371394},
        {-0.939499,  0.069011, -0.335529}, {-0.969804,  0.073461, -0.232558},
        {-0.946917, -0.096998, -0.306496}, {-0.976435, -0.165058, -0.139033},
        {-0.974580, -0.089811, -0.205249}, {-0.985924, -0.005978, -0.167088},
        {-0.053324,  0.964447, -0.258841}, { 0.016203,  0.915993, -0.400866},
        {-0.070546,  0.936889, -0.342435}, {-0.173326,  0.915993, -0.361821},
        { 0.083434,  0.841980, -0.533018}, { 0.145350,  0.747137, -0.648582},
        { 0.064727,  0.791053, -0.608314}, {-0.035501,  0.772328, -0.634231},
        {-0.000945,  0.876033, -0.482250}, {-0.119927,  0.804197, -0.582139},
        {-0.104085,  0.856678, -0.505241}, {-0.189740,  0.876033, -0.443356},
        {-0.218068,  0.772328, -0.596620}, {-0.389865,  0.747138, -0.538322},
        {-0.299891,  0.791053, -0.533199}, {-0.287315,  0.841980, -0.456640},
        { 0.005662, -0.885991, -0.463667}, { 0.129121, -0.825694, -0.549142},
        { 0.104696, -0.874305, -0.473952}, { 0.177730, -0.898184, -0.402092},
        { 0.253129, -0.742154, -0.620591}, { 0.371337, -0.639244, -0.673406},
        { 0.352945, -0.704953, -0.615200}, { 0.429056, -0.722221, -0.542501},
        { 0.230200, -0.800706, -0.553062}, { 0.405447, -0.780851, -0.475274},
        { 0.306298, -0.821910, -0.480256}, { 0.278310, -0.871092, -0.404650},
        { 0.473263, -0.786703, -0.396384}, { 0.502466, -0.829703, -0.243147},
        { 0.443147, -0.835958, -0.323721}, { 0.347195, -0.878983, -0.326871},
        {-0.557655, -0.829704, -0.024750}, {-0.591397, -0.786703, -0.177053},
        {-0.535013, -0.835958, -0.122209}, {-0.448119, -0.878983, -0.163028},
        {-0.608542, -0.722222, -0.328744}, {-0.607263, -0.639244, -0.471804},
        {-0.567363, -0.704953, -0.425606}, {-0.477805, -0.742154, -0.470010},
        {-0.560284, -0.780851, -0.276323}, {-0.430052, -0.800706, -0.417043},
        {-0.471177, -0.821910, -0.320087}, {-0.415585, -0.871092, -0.261700},
        {-0.335654, -0.825695, -0.453393}, {-0.188464, -0.885991, -0.423675},
        {-0.283499, -0.874305, -0.393979}, {-0.322184, -0.898184, -0.299105},
        {-0.953363, -0.249821, -0.169376}, {-0.923809, -0.181892, -0.336886},
        {-0.928397, -0.259481, -0.265986}, {-0.894196, -0.342972, -0.287722},
        {-0.861473, -0.107892, -0.496208}, {-0.770620, -0.032883, -0.636446},
        {-0.803902, -0.111690, -0.584180}, {-0.767608, -0.192697, -0.611266},
        {-0.882075, -0.187484, -0.432196}, {-0.790242, -0.272874, -0.548687},
        {-0.846840, -0.271403, -0.457387}, {-0.854704, -0.349398, -0.383930},
        {-0.744783, -0.350916, -0.567588}, {-0.703004, -0.498942, -0.506796},
        {-0.756413, -0.427913, -0.494702}, {-0.809496, -0.427738, -0.402190},
        { 0.431392,  0.137040, -0.891696}, { 0.398047,  0.314627, -0.861724},
        { 0.457871,  0.228124, -0.859251}, { 0.537458,  0.229271, -0.811526},
        { 0.351773,  0.484629, -0.800869}, { 0.295735,  0.635419, -0.713291},
        { 0.365455,  0.567647, -0.737712}, { 0.443487,  0.574503, -0.687943},
        { 0.417621,  0.404861, -0.813437}, { 0.510541,  0.496059, -0.702334},
        { 0.498210,  0.409243, -0.764399}, { 0.558394,  0.320257, -0.765266},
        { 0.582445,  0.495343, -0.644511}, { 0.704780,  0.401265, -0.585040},
        { 0.642400,  0.407254, -0.649204}, { 0.630483,  0.318858, -0.707687},
        { 0.489504,  0.051880, -0.870456}, { 0.595443,  0.144298, -0.790333},
        { 0.573977,  0.057979, -0.816816}, { 0.628440, -0.022998, -0.777518},
        { 0.688655,  0.233611, -0.686426}, { 0.763951,  0.314554, -0.563414},
        { 0.759240,  0.236034, -0.606500}, { 0.813471,  0.156790, -0.560073},
        { 0.674586,  0.149986, -0.722799}, { 0.798919,  0.073461, -0.596935},
        { 0.730382,  0.069011, -0.679543}, { 0.705928, -0.017632, -0.708064},
        { 0.839603, -0.005978, -0.543167}, { 0.841976, -0.165058, -0.513646},
        { 0.814100, -0.089811, -0.573738}, { 0.748672, -0.096998, -0.655807},
        { 0.456318, -0.032883, -0.889209}, { 0.595202, -0.107892, -0.796300},
        { 0.507548, -0.111690, -0.854353}, { 0.463503, -0.192697, -0.864889},
        { 0.715434, -0.181892, -0.674589}, { 0.808789, -0.249821, -0.532399},
        { 0.747671, -0.259481, -0.611275}, { 0.707664, -0.342972, -0.617723},
        { 0.639427, -0.187484, -0.745642}, { 0.633362, -0.349398, -0.690488},
        { 0.597104, -0.271403, -0.754856}, { 0.509028, -0.272874, -0.816352},
        { 0.584618, -0.427737, -0.689393}, { 0.445452, -0.498942, -0.743391},
        { 0.499292, -0.427913, -0.753391}, { 0.459801, -0.350916, -0.815746},
        {-0.276003,  0.309387, -0.910001}, {-0.425849,  0.252840, -0.868749},
        {-0.354847,  0.332267, -0.873889}, {-0.356751,  0.429325, -0.829704},
        {-0.564808,  0.191210, -0.802764}, {-0.685915,  0.127466, -0.716431},
        {-0.634517,  0.208961, -0.744126}, {-0.643341,  0.305039, -0.702184},
        {-0.502148,  0.272499, -0.820727}, {-0.579301,  0.385678, -0.718097},
        {-0.507597,  0.370796, -0.777725}, {-0.433318,  0.448301, -0.781832},
        {-0.576783,  0.475041, -0.664573}, {-0.490790,  0.625845, -0.606170},
        {-0.501588,  0.548484, -0.669010}, {-0.429643,  0.536442, -0.726386},
        {-0.187471,  0.369801, -0.910001}, {-0.269715,  0.488718, -0.829704},
        {-0.180031,  0.451560, -0.873889}, {-0.080186,  0.488718, -0.868749},
        {-0.342885,  0.595645, -0.726386}, {-0.403843,  0.685178, -0.606170},
        {-0.327870,  0.667028, -0.669010}, {-0.232067,  0.710273, -0.664573},
        {-0.259470,  0.566933, -0.781832}, {-0.147936,  0.680038, -0.718097},
        {-0.160220,  0.607844, -0.777725}, {-0.070675,  0.566933, -0.820726},
        {-0.049500,  0.710273, -0.702184}, { 0.131372,  0.685178, -0.716431},
        { 0.036748,  0.667028, -0.744125}, { 0.027863,  0.595645, -0.802764},
        {-0.135742, -0.436900, -0.889209}, { 0.010454, -0.501854, -0.864889},
        {-0.081035, -0.513335, -0.854354}, {-0.116528, -0.593572, -0.796300},
        { 0.159133, -0.556089, -0.815746}, { 0.302204, -0.596693, -0.743391},
        { 0.216434, -0.620933, -0.753391}, { 0.185164, -0.700322, -0.689393},
        { 0.068516, -0.573476, -0.816352}, { 0.094445, -0.717151, -0.690488},
        { 0.035036, -0.654954, -0.754856}, {-0.058537, -0.663770, -0.745643},
        { 0.061373, -0.783997, -0.617723}, {-0.062235, -0.844202, -0.532399},
        {-0.030958, -0.790812, -0.611275}, {-0.091454, -0.732507, -0.674589},
        {-0.226770, -0.436900, -0.870456}, {-0.207697, -0.593572, -0.777518},
        {-0.263245, -0.513335, -0.816816}, {-0.351449, -0.501854, -0.790333},
        {-0.182623, -0.732507, -0.655807}, {-0.153263, -0.844202, -0.513647},
        {-0.213168, -0.790812, -0.573738}, {-0.300529, -0.783997, -0.543167},
        {-0.240944, -0.663770, -0.708065}, {-0.359669, -0.717152, -0.596935},
        {-0.330539, -0.654954, -0.679543}, {-0.385598, -0.573477, -0.722799},
        {-0.442567, -0.700323, -0.560073}, {-0.571420, -0.596694, -0.563414},
        {-0.496587, -0.620933, -0.606501}, {-0.468598, -0.556089, -0.686426},
        {-0.284882, -0.351740, -0.891696}, {-0.409434, -0.416881, -0.811526},
        {-0.379351, -0.343190, -0.859252}, {-0.438090, -0.255946, -0.861724},
        {-0.526769, -0.470843, -0.707687}, {-0.630591, -0.509983, -0.585041},
        {-0.613427, -0.449713, -0.649204}, {-0.673594, -0.361769, -0.644511},
        {-0.501791, -0.403205, -0.765266}, {-0.648047, -0.294553, -0.702334},
        {-0.562711, -0.314722, -0.764400}, {-0.529250, -0.241277, -0.813437},
        {-0.696646, -0.203516, -0.687943}, {-0.699503, -0.043725, -0.713291},
        {-0.661814, -0.133354, -0.737712}, {-0.579521, -0.150880, -0.800868},
        { 0.346897,  0.127467, -0.929201}, { 0.201528,  0.191210, -0.960638},
        { 0.288738,  0.208962, -0.934326}, { 0.313420,  0.305039, -0.899288},
        { 0.047802,  0.252840, -0.966326}, {-0.106147,  0.309387, -0.944993},
        {-0.019450,  0.332267, -0.942985}, {-0.000236,  0.429325, -0.903150},
        { 0.136870,  0.272499, -0.952371}, { 0.089017,  0.448301, -0.889439},
        { 0.158871,  0.370796, -0.915025}, { 0.248305,  0.385678, -0.888593},
        { 0.107557,  0.536442, -0.837055}, { 0.211240,  0.625845, -0.750796},
        { 0.196321,  0.548484, -0.812787}, { 0.267147,  0.475041, -0.838432},
        { 0.348009, -0.509982, -0.786643}, { 0.204165, -0.470843, -0.858268},
        { 0.306882, -0.449713, -0.838798}, { 0.364005, -0.361769, -0.858268},
        { 0.055341, -0.416881, -0.907275}, {-0.090756, -0.351740, -0.931688},
        { 0.008845, -0.343190, -0.939224}, { 0.061824, -0.255946, -0.964712},
        { 0.158462, -0.403205, -0.901286}, { 0.164646, -0.241277, -0.956387},
        { 0.214764, -0.314722, -0.924568}, { 0.317683, -0.294553, -0.901286},
        { 0.215792, -0.150880, -0.964712}, { 0.360619, -0.043725, -0.931688},
        { 0.316346, -0.133354, -0.939224}, { 0.368013, -0.203516, -0.907275},
        {-0.249395,  0.211636, -0.944993}, {-0.252866,  0.047667, -0.966326},
        {-0.302308,  0.139247, -0.942985}, {-0.399691,  0.156740, -0.903150},
        {-0.251522, -0.117947, -0.960638}, {-0.245163, -0.276550, -0.929201},
        {-0.299846, -0.192683, -0.934326}, {-0.398310, -0.180640, -0.899288},
        {-0.303643, -0.028104, -0.952371}, {-0.449659, -0.090608, -0.888593},
        {-0.403196, -0.012755, -0.915025}, {-0.449900,  0.080548, -0.889439},
        {-0.539741, -0.075573, -0.838432}, {-0.659784,  0.031464, -0.750796},
        {-0.582308,  0.017153, -0.812787}, {-0.538734,  0.095418, -0.837055},
        {-0.142065, -0.276550, -0.950441}, {-0.148649, -0.117947, -0.981831},
        {-0.093860, -0.192683, -0.976762}, { 0.010435, -0.180640, -0.983494},
        {-0.149663,  0.047667, -0.987587}, {-0.144419,  0.211636, -0.966620},
        {-0.095021,  0.139247, -0.985688}, { 0.010177,  0.156740, -0.987587},
        {-0.097504, -0.028104, -0.994838}, { 0.061717,  0.080548, -0.994838},
        { 0.008704, -0.012755, -0.999881}, { 0.061830, -0.090608, -0.993965},
        { 0.164023,  0.095418, -0.981831}, { 0.309310,  0.031464, -0.950441},
        { 0.213641,  0.017153, -0.976762}, { 0.164403, -0.075573, -0.983494}};

//static double RR = 1.9858775; // cal/mol
//static double TT = 300.00;    // Kevin

/*        Bond angle parameters             */
static double AngleCACN = 116.2*PI/180.0;
static double AngleCNCA = 121.7*PI/180.0;
static double AngleNCAC = 111.2*PI/180.0;
static double AngleCACBSG = 108.6*PI/180.0;
static double AngleCACBOG = 109.5*PI/180.0;
static double AngleCACBCG = 109.5*PI/180.0;
static double AngleCBCGCD = 109.5*PI/180.0;
static double AngleCGCDCE = 109.5*PI/180.0;
static double AngleCACBCG_ASN = 111.1*PI/180.0;
static double AngleCBCGOD1_ASN = 120.4*PI/180.0;
static double AngleCBCGND2_ASN = 116.6*PI/180.0;
static double AngleCACBCG_ASP = 111.1*PI/180.0;
static double AngleCBCGOD1_ASP = 117.0*PI/180.0;
static double AngleCBCGOD2_ASP = 117.0*PI/180.0;
static double AngleCBCGSD = 114.7*PI/180.0;
static double AngleCGSDCE = 98.9*PI/180.0;
static double AngleCBCGCD_GLU = 111.1*PI/180.0;
static double AngleCGCDOE1_GLU = 117.0*PI/180.0;
static double AngleCGCDOE2_GLU = 117.0*PI/180.0;
static double AngleCBCGCD_GLN = 111.1*PI/180.0;
static double AngleCGCDOE1_GLN = 120.4*PI/180.0;
static double AngleCGCDNE2_GLN = 116.6*PI/180.0;
static double AngleCDCENZ = 111.2*PI/180.0;
static double AngleCACBCG_HIS = 113.1*PI/180.0;
static double AngleCBCGND1_HIS = 120.0*PI/180.0;
static double AngleCBCGCD2_HIS = 120.0*PI/180.0;
static double AngleCGND1CE1_HIS = 108.0*PI/180.0;
static double AngleCGCD2NE2_HIS = 108.0*PI/180.0;
static double AngleCTCTCA = 114.0*PI/180.0;
static double AngleCTCACA = 120.0*PI/180.0;
static double AngleCACACA = 120.0*PI/180.0;
static double AngleCTCTCT = 109.5*PI/180.0;
static double AngleCTCTN2 = 111.2*PI/180.0;
static double AngleCTN2CA = 123.2*PI/180.0;
static double AngleN2CAN2 = 120.0*PI/180.0;
static double AngleCACAC  = 120.0*PI/180.0;
static double AngleCACOH  = 120.0*PI/180.0;
static double AngleCTCTC_  = 120.0*PI/180.0;
static double AngleCTC_CW  = 125.0*PI/180.0;
static double AngleCTC_CB  = 128.6*PI/180.0;
static double AngleC_CWNA  = 108.7*PI/180.0;
static double AngleC_CBCN  = 108.8*PI/180.0;
static double AngleC_CBCA  = 134.9*PI/180.0;
static double AngleCBCNCA  = 122.7*PI/180.0;
static double AngleCBCACA  = 120.0*PI/180.0;
static double AngleCNCACA  = 120.0*PI/180.0;
static double AngleCTCTCT_PRO = 108.0*PI/180;

/*        Bond length parameters            */
static double BondCN    = 1.329;
static double BondNCA   = 1.458;
static double BondCAC   = 1.525;
static double BondCO    = 1.231; // bond length of C=O
static double BondNH    = 1.01;  // bond length of N-H
static double BondCACB  = 1.526; // bond length of CA-CB
static double BondCBSG  = 1.810; // bond length of CB-SG
static double BondCBOG  = 1.410; // bond length of CB-OG
static double BondCBCG  = 1.526; // bond length of CB-CG
static double BondCGCD  = 1.526; // bond length of CG-CD
static double BondCDCE  = 1.526; // bond length of CD-CE
static double BondCBCG_ASN   = 1.522; // bond length of CB-CG ASN
static double BondCGOD1_ASN  = 1.229; // bond length of CB-CG ASN
static double BondCGND2_ASN  = 1.335; // bond length of CB-CG ASN
static double BondCBCG_ASP   = 1.522; // bond length of CB-CG ASP
static double BondCGOD1_ASP  = 1.250; // bond length of CB-CG ASP
static double BondCGOD2_ASP  = 1.250; // bond length of CB-CG ASP
static double BondCGSD  = 1.810; // bond length of CG-SD MET
static double BondSDCE  = 1.810; // bond length of SD-CE MET
static double BondCGCD_GLU   = 1.522; 
static double BondCDOE1_GLU  = 1.250; 
static double BondCDOE2_GLU  = 1.250; 
static double BondCGCD_GLN   = 1.522;
static double BondCDOE1_GLN  = 1.229;
static double BondCDNE2_GLN  = 1.335;
static double BondCENZ  = 1.471; // bond length of CE-NZ
static double BondCBCG_HIS  = 1.504;
static double BondCGND1_HIS  = 1.385;
static double BondCGCD2_HIS  = 1.375;
static double BondND1CE1_HIS  = 1.343;
static double BondCD2NE2_HIS  = 1.394;
static double BondCTCA  = 1.510;
static double BondCACA  = 1.400;
static double BondCTCT  = 1.526;
static double BondCTN2  = 1.463;
static double BondCAN2  = 1.340;
static double BondCCA   = 1.409;
static double BondCOH   = 1.364;
static double BondCTC_  = 1.495;
static double BondC_CW  = 1.352;
static double BondC_CB  = 1.459;
static double BondCWNA  = 1.381;
static double BondCBCN  = 1.364;
static double BondCBCA  = 1.419;
static double BondCNCA  = 1.400;

